from kitconcept.solr import PACKAGE_NAME

import pytest


class TestSetupUninstall:
    @pytest.fixture(autouse=True)
    def uninstalled(self, installer):
        installer.uninstall_product(PACKAGE_NAME)

    def test_product_uninstalled(self, installer):
        """Test if kitconcept.solr is cleanly uninstalled."""
        assert installer.is_product_installed(PACKAGE_NAME) is False

    def test_browserlayer_removed(self, browser_layers):
        """Test that IKitconceptSolrLayer is removed."""
        from kitconcept.solr.interfaces import IKitconceptSolrLayer

        assert IKitconceptSolrLayer not in browser_layers



import pytest


solr_config = {
    "fieldList": [
        "UID",
        "Title",
        "Description",
        "Type",
        "effective",
        "start",
        "created",
        "end",
        "path_string",
        "phone",
        "email",
        "location",
    ],
    "searchTabs": [
        {
            "label": "All",
            "filter": "Type(*)",
        },
        {
            "label": "Pages",
            "filter": "Type:(Page)",
        },
        {
            "label": "News Items",
            "filter": 'Type:("News Item")',
        },
        {
            "label": "Pages and News Items",
            "filter": 'Type:(Page OR "News Item")',
        },
    ],
}


@pytest.fixture()
def registry_config() -> dict:
    """Override registry configuration."""
    return {
        "collective.solr.active": 1,
        "kitconcept.solr.config": solr_config,
    }


class TestResponseCustom:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content, manager_request):
        self.portal = portal_with_content
        response = manager_request.get(self.url)
        self.data = response.json()


class TestResponseKeepFullSolrResponseDefault(TestResponseCustom):
    url = "/@solr?q=chomsky"

    def test_facet_counts(self):
        assert "response" in self.data
        assert "facet_counts" not in self.data


class TestResponseKeepFullSolrResponseIsFalse(TestResponseCustom):
    url = "/@solr?q=chomsky&keep_full_solr_response=false"

    def test_facet_counts(self):
        assert "response" in self.data
        assert "facet_counts" not in self.data


class TestResponseKeepFullSolrResponseDefaultIsTrue(TestResponseCustom):
    url = "/@solr?q=chomsky&keep_full_solr_response=true"

    def test_facet_counts(self):
        assert "response" in self.data
        assert "facet_counts" in self.data



from plone import api
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles:
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller"""
        return [
            "kitconcept.solr:uninstall",
        ]


def post_install(context):
    """Post install script"""


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.


def add_catalog_indexes(context, wanted=None):
    """Method to add our wanted indexes to the portal_catalog."""
    catalog = api.portal.get_tool("portal_catalog")
    indexes = catalog.indexes()
    indexables = []
    for name, meta_type in wanted:
        if name not in indexes:
            catalog.addIndex(name, meta_type)
            indexables.append(name)
    if len(indexables) > 0:
        catalog.manage_reindexIndex(ids=indexables)



from collective.solr.interfaces import ISolrConnectionManager
from plone import api
from zope.component import queryUtility

import logging


logger = logging.getLogger("kitconcept.solr")
logger.setLevel(logging.DEBUG)

indexer_logger = logging.getLogger("collective.solr.indexer")


def solr_is_running(portal):
    manager = queryUtility(ISolrConnectionManager, context=portal)
    schema = manager.getSchema()
    return schema is not None


def solr_must_be_running(portal):
    if not solr_is_running(portal):
        logger.fatal("*** Solr must be running! (make solr-start) ***")
        return False
    return True


def activate(active=True):
    """(de)activate the solr integration"""
    api.portal.set_registry_record("collective.solr.active", active)


def silence_logger():
    orig_logger_exception = indexer_logger.exception

    def new_logger_exception(msg):
        if msg != "Error occured while getting data for indexing!":
            orig_logger_exception(msg)

    indexer_logger.exception = new_logger_exception

    def reactivate_logger():
        indexer_logger.exception = orig_logger_exception

    return reactivate_logger


def reindex(portal, clear=False):
    """reindex the existing content in solr"""
    maintenance = portal.unrestrictedTraverse("@@solr-maintenance")
    if clear:
        logger.info("Clearing solr...")
        maintenance.clear()
    # Avoid throwing a lot of errors which are actually not errors,
    # but the indexer keeps throwing them when it tries to traverse everything.
    reactivate_logger = silence_logger()
    logger.info("Reindexing solr...")
    maintenance.reindex()
    reactivate_logger()


def activate_and_reindex(portal, clear=False):
    # Activate before confirming solr is running,
    # because the confirmation only works if solr is enabled in the registry.
    # If solr isn't running, we'll exit
    # before committing the transaction with the activation.
    activate()
    if solr_must_be_running(portal):
        reindex(portal, clear=clear)



from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD

import pytest


@pytest.fixture
def contents() -> list:
    return [
        {
            "_container": "",
            "type": "Image",
            "id": "noamchomsky",
            "title": "Prof. Dr. Noam Chomsky",
            "subjects": ["mymembersubject", "mymembersubjecttwo"],
            "_image": b"iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAAAXNSR0IArs4c6QAAAA1JREFUGFdjCDO+/R8ABKsCZD++CcMAAAAASUVORK5CYII=",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "chomskysdocument",
            "title": "Noam Chomsky's private document",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "mydocument",
            "title": "My Document about Noam Chomsky",
            "_transitions": ["publish"],
        },
        {
            "_container": "",
            "type": "News Item",
            "id": "mynews",
            "title": "My News Item with Noam Chomsky",
            "_transitions": ["publish"],
        },
        {
            "_container": "",
            "type": "Document",
            "id": "myotherfolder",
            "title": "My other Folder",
            "_transitions": ["publish"],
        },
    ]


@pytest.fixture
def user_credentials() -> tuple:
    return "user2", "averylongpasswordbutnotthatlong"


@pytest.fixture
def users(user_credentials) -> list:
    return [
        {
            "username": user_credentials[0],
            "email": "user2@foo.bar",
            "password": user_credentials[1],
            "roles": ["Member"],
        },
    ]


@pytest.fixture
def users_credentials_role(user_credentials) -> dict:
    return {
        "manager": (SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        "member": user_credentials,
        "anonymous": None,
    }


solr_config = {
    "fieldList": [
        "UID",
        "Title",
        "Description",
        "Type",
        "effective",
        "start",
        "created",
        "end",
        "path_string",
        "phone",
        "email",
        "location",
    ],
    "searchTabs": [
        {
            "label": "All",
            "filter": "Type(*)",
        },
        {
            "label": "Pages",
            "filter": "Type:(Page)",
        },
        {
            "label": "News Items",
            "filter": 'Type:("News Item")',
        },
        {
            "label": "Images and News Items",
            "filter": 'Type:(Image OR "News Item")',
        },
        {
            "label": "Pages and News Items",
            "filter": 'Type:(Page OR "News Item")',
        },
        {
            "label": "Contacts",
            "filter": "Type:(Image)",
        },
    ],
}


@pytest.fixture()
def registry_config() -> dict:
    """Override registry configuration."""
    return {
        "collective.solr.active": 1,
        "kitconcept.solr.config": solr_config,
    }



import pytest


@pytest.fixture
def contents() -> list:
    return [
        {
            "_container": "",
            "type": "Image",
            "id": "myimage",
            "title": "Prof. Dr. Noam Chomsky",
            "subjects": ["mymembersubject", "mymembersubjecttwo"],
            "_image": b"iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAAAXNSR0IArs4c6QAAAA1JREFUGFdjCDO+/R8ABKsCZD++CcMAAAAASUVORK5CYII=",
            "language": "en",
        },
        {
            "_container": "",
            "type": "Folder",
            "id": "myfolder",
            "title": "My Folder to store everything about Noam Chomsky",
            "language": "en",
        },
        {
            "_container": "myfolder",
            "type": "News Item",
            "id": "mynews",
            "title": "My News Item with Noam Chomsky",
            "language": "en",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "foo_alpha",
            "title": "Chomsky foo",
            "description": "Chomsky alpha",
            "language": "en",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "foo_beta",
            "title": "Chomsky foo",
            "description": "Chomsky beta",
            "language": "en",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "foo_gamma",
            "title": "Chomsky foo",
            "description": "Chomsky gamma",
            "language": "en",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "bar_alpha",
            "title": "Chomsky bar",
            "description": "Chomsky alpha",
            "language": "en",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "bar_beta",
            "title": "Chomsky bar",
            "description": "Chomsky beta",
            "language": "en",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "bar_gamma",
            "title": "Chomsky bar",
            "description": "Chomsky gamma",
            "language": "en",
        },
    ]



import pytest


class TestEndpointPortalType:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content, manager_request):
        self.portal = portal_with_content
        response = manager_request.get(self.url)
        self.data = response.json()


class TestEndpointPortalTypeNotPassed(TestEndpointPortalType):
    url = "/@solr?q=news"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/redandblue", False),
            ("/plone/news1", True),
            ("/plone/news2", True),
            ("/plone/news3", True),
            ("/plone/blue", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointPortalTypeSingle(TestEndpointPortalType):
    url = "/@solr?q=blue&portal_type:list=News Item"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/redandblue", False),
            ("/plone/news1", False),
            ("/plone/news2", False),
            ("/plone/news3", False),
            ("/plone/blue", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointPortalTypeMultiple(TestEndpointPortalType):
    url = "/@solr?q=blue&portal_type:list=News Item&portal_type:list=Page"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/redandblue", True),
            ("/plone/news1", False),
            ("/plone/news2", False),
            ("/plone/news3", False),
            ("/plone/blue", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointPortalTypeMultipleWithSpace(TestEndpointPortalType):
    url = "/@solr?q=blue&portal_type:list=News Item&portal_type:list=News%20Item"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/redandblue", False),
            ("/plone/news1", False),
            ("/plone/news2", False),
            ("/plone/news3", False),
            ("/plone/blue", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected



import pytest


@pytest.fixture
def contents() -> list:
    return [
        {
            "_container": "",
            "type": "Image",
            "id": "noamchomsky",
            "title": "Prof. Dr. Noam Chomsky",
            "subjects": ["mymembersubject", "mymembersubjecttwo"],
            "_image": b"iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAAAXNSR0IArs4c6QAAAA1JREFUGFdjCDO+/R8ABKsCZD++CcMAAAAASUVORK5CYII=",
        },
        {
            "_container": "",
            "type": "Folder",
            "id": "myfolder",
            "title": "My Folder to store everything about Noam Chomsky",
        },
        {
            "_container": "",
            "type": "Folder",
            "id": "myotherfolder",
            "title": "My other Folder",
        },
        {
            "_container": "myfolder",
            "type": "Document",
            "id": "mydocument",
            "title": "My Document about Noam Chomsky",
        },
        {
            "_container": "myfolder",
            "type": "News Item",
            "id": "mynews",
            "title": "My News Item with Noam Chomsky",
        },
    ]


class TestEndpointLocalSearch:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content, manager_request):
        self.portal = portal_with_content
        response = manager_request.get(self.url)
        self.data = response.json()


class TestEndpointLocalSearchGlobal(TestEndpointLocalSearch):
    """Search without a prefix."""

    url = "/@solr?q=chomsky"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", True),
            ("/plone/myfolder/mydocument", True),
            ("/plone/myfolder/mynews", True),
            ("/plone/myfolder", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointLocalSearchPrefix(TestEndpointLocalSearch):
    """Search all matching objects in a folder, including the parent."""

    url = "/@solr?q=chomsky&path_prefix=/myfolder"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", False),
            ("/plone/myfolder/mydocument", True),
            ("/plone/myfolder/mynews", True),
            ("/plone/myfolder", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointLocalSearchContext(TestEndpointLocalSearch):
    """Search all matching objects in a folder, including the parent."""

    url = "/myfolder/@solr?q=chomsky"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", False),
            ("/plone/myfolder/mydocument", True),
            ("/plone/myfolder/mynews", True),
            ("/plone/myfolder", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointLocalSearchContextWithPrefix(TestEndpointLocalSearch):
    """Search all matching objects in a folder, including the parent.
    Called on object context with path_prefix which overrides the context.
    """

    url = "/myotherfolder/@solr?q=chomsky&path_prefix=/myfolder"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", False),
            ("/plone/myfolder/mydocument", True),
            ("/plone/myfolder/mynews", True),
            ("/plone/myfolder", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected

    def test_portal_path(self):
        """portal_path depends on the portal path, independent from context"""
        assert "response" in self.data
        assert self.data.get("portal_path") == "/plone"


class TestEndpointLocalSearchPrefixExcludeParent(TestEndpointLocalSearch):
    """Search all matching objects in a folder, excluding the parent."""

    url = "/@solr?q=chomsky&path_prefix=/myfolder/"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", False),
            ("/plone/myfolder/mydocument", True),
            ("/plone/myfolder/mynews", True),
            ("/plone/myfolder", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointLocalSearchSiteRoot(TestEndpointLocalSearch):
    """Search all matching objects in the site."""

    url = "/@solr?q=chomsky&path_prefix=/"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", True),
            ("/plone/myfolder/mydocument", True),
            ("/plone/myfolder/mynews", True),
            ("/plone/myfolder", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected



from plone.indexer import indexer
from plone.restapi.behaviors import IBlocks
from plone.restapi.interfaces import IBlockSearchableText
from typing import Any
from zope.component import queryMultiAdapter
from zope.globalrequest import getRequest


def safe_dict_traverse(data: dict, path: str, default: Any) -> Any:
    """Safe traverse a dict based on a path.

    :param data: Dictionary containing the data.
    :param path: Path to the value. i.e. about/value/0/children/text
    :param default: Default value, case the traverse fails.
    :returns: Value extracted from data.
    """
    error = False
    path = path[1:] if path.startswith("/") else path
    parts = path.split("/")
    value = default
    tmp_data = data
    for idx in parts:
        try:
            if isinstance(tmp_data, dict):
                tmp_data = tmp_data[idx]
            elif isinstance(tmp_data, list):
                tmp_data = tmp_data[int(idx)]
        except (KeyError, IndexError, ValueError):
            # Either the key does not exist in the dictionary
            # Or the index does not exist in the list
            # Or we have a list and the index is not a number
            error = True
            break
    if not error:
        value = tmp_data
    return value


def _extract_block_text(block: dict) -> str:
    """Extract text from a block.

    :param block: Dictionary with block information.
    :returns: A string with text found in the block.
    """
    block_type = block.get("@type", "")
    result = ""
    match block_type:
        case "headline":
            result = block.get("title", "")
        case "introduction":
            about = safe_dict_traverse(block, "about/value/0/children/0/text", "")
            topics = safe_dict_traverse(block, "topics/value/0/children/0/text", "")
            result = f"{about}\n{topics}"
        case "slateTable":
            for row in range(len(block["table"]["rows"])):
                for column in range(len(block["table"]["rows"][row])):
                    path = f"table/rows/{row}/cells/{column}/value/0/children/0/text"
                    cell = safe_dict_traverse(block, path, "")
                    result = f"{result}\n{cell}"
        case "tabBlock":
            tab = ""
            tab_amount = len(block["columns"])
            for i in range(tab_amount):
                path = f"text-{i}/blocks/0/text"
                tab = safe_dict_traverse(block, path, "")
                result = f"{result}\n{tab}"
    return result


def extract_text(block, obj, request):
    """Extract text information from a block.
    This function tries the following methods, until it finds a result:
        1. searchableText attribute
        2. Server side adapter
        3. Subblocks
    The decision to use the server side adapter before the subblocks traversal
    allows addon developers to choose this implementation when they want a
    more granular control of the indexing.
    :param block: Dictionary with block information.
    :param obj: Context to be used to get a IBlockSearchableText.
    :param request: Current request.
    :returns: A string with text found in the block.
    """
    result = ""
    block_type = block.get("@type", "")
    # searchableText is the conventional way of storing
    # searchable info in a block
    if searchableText := block.get("searchableText", ""):
        # TODO: should we evaluate in some way this value? maybe passing
        # it into html/plain text transformer?
        return searchableText
    elif plaintext := block.get("plaintext", ""):
        return plaintext
    elif text := block.get("text", ""):
        # Some blocks have the text attribute directly available (no richtext)
        return text

    if result := _extract_block_text(block):
        return result

    # Use server side adapters to extract the text data
    adapter = queryMultiAdapter((obj, request), IBlockSearchableText, name=block_type)
    result = adapter(block) if adapter is not None else ""
    if not result:
        subblocks = extract_subblocks(block)
        for subblock in subblocks:
            tmp_result = extract_text(subblock, obj, request)
            result = f"{result}\n{tmp_result}"
    return result


def extract_subblocks(block):
    """Extract subblocks from a block.
    :param block: Dictionary with block information.
    :returns: A list with subblocks, if present, or an empty list.
    """
    if "data" in block and "blocks" in block["data"]:
        raw_blocks = block["data"]["blocks"]
    elif "blocks" in block:
        raw_blocks = block["blocks"]
    else:
        raw_blocks = None
    if "columns" in block:
        # Check for grid block
        if block["@type"] == "__grid":
            columns = block["columns"]
            return columns
        return []
    return list(raw_blocks.values()) if isinstance(raw_blocks, dict) else []


@indexer(IBlocks)
def body_text_blocks(obj):
    """Index text to be used with Solr."""
    request = getRequest()
    blocks = obj.blocks
    blocks_layout = obj.blocks_layout
    blocks_text = []
    for block_id in blocks_layout.get("items", []):
        block = blocks.get(block_id, {})
        blocks_text.append(extract_text(block, obj, request))

    return " ".join([text.strip() for text in blocks_text if text.strip()])



from kitconcept.solr.services.solr_utils import replace_reserved


class TestUtilsReplaceReserved:
    def test_replace_reserved_and(self):
        assert replace_reserved("foo AND bar") == "foo and bar"

    def test_replace_reserved_or(self):
        assert replace_reserved("foo OR bar") == "foo or bar"

    def test_replace_reserved_not(self):
        assert replace_reserved("foo NOT bar") == "foo not bar"

    def test_replace_reserved_keep_lowercase(self):
        assert replace_reserved("foo AnD NoT bar Or OReo") == "foo AnD NoT bar Or OReo"

    def test_replace_reserved_keep_partial_match(self):
        assert replace_reserved("OReo ANDroid kNOT") == "OReo ANDroid kNOT"



from kitconcept.solr.testing import FUNCTIONAL_TESTING
from kitconcept.solr.testing import INTEGRATION_TESTING
from pathlib import Path
from pytest_plone import fixtures_factory
from requests import exceptions as exc

import pytest
import requests


pytest_plugins = ["pytest_plone"]


globals().update(
    fixtures_factory((
        (FUNCTIONAL_TESTING, "functional"),
        (INTEGRATION_TESTING, "integration"),
    ))
)


def is_responsive(url):
    """Helper fixture to check if Solr is up and running."""
    try:
        response = requests.get(url, timeout=5)
        if response.status_code == 200:
            return b"""<str name="status">OK</str>""" in response.content
    except (exc.ConnectionError, exc.Timeout):
        return False


@pytest.fixture(scope="session")
def docker_compose_project_name() -> str:
    """Return the name of the Docker Compose project."""
    return "kitconcept-solr-tests"


@pytest.fixture(scope="session")
def docker_setup():
    """Return the Docker Compose commands to set up the stack."""
    # Stop the stack before starting a new one, only start the Solr service
    profile = "solr"
    return [f"--profile {profile} down -v", f"--profile {profile} up --build -d"]


@pytest.fixture(scope="session")
def docker_compose_file(pytestconfig):
    """Fixture pointing to the docker-compose file to be used."""
    backend_root = Path(str(pytestconfig.rootdir)).resolve()
    repo_root = backend_root.parent
    return repo_root / "docker-compose-dev.yml"


@pytest.fixture
def solr_service(docker_ip, docker_services):
    """Ensure that Solr service is up and responsive."""
    port = docker_services.port_for("solr-acceptance", 8983)
    url = f"http://{docker_ip}:{port}/solr/plone/admin/ping?wt=xml"
    docker_services.wait_until_responsive(
        timeout=90.0, pause=0.1, check=lambda: is_responsive(url)
    )
    return url



from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.restapi.testing import PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

import kitconcept.solr


class KSolrLayer(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        self.loadZCML(package=kitconcept.solr)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "kitconcept.solr:default")
        applyProfile(portal, "kitconcept.solr:testing")


FIXTURE = KSolrLayer()


INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,),
    name="KSolrLayer:IntegrationTesting",
)


FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE, PLONE_RESTAPI_DX_FUNCTIONAL_TESTING),
    name="KSolrLayer:FunctionalTesting",
)



from kitconcept.solr import PACKAGE_NAME
from plone import api

import pytest


class TestSetupInstall:
    def test_addon_installed(self, installer):
        assert installer.is_product_installed(PACKAGE_NAME) is True

    def test_browserlayer(self, browser_layers):
        """Test that IKitconceptSolrLayer is registered."""
        from kitconcept.solr.interfaces import IKitconceptSolrLayer

        assert IKitconceptSolrLayer in browser_layers

    @pytest.mark.parametrize("package_name", ["collective.solr"])
    def test_dependency_installed(self, installer, package_name):
        """Test dependencies are installed."""
        assert installer.is_product_installed(package_name) is True

    @pytest.mark.parametrize(
        "key,expected",
        [
            ["collective.solr.host", "127.0.0.1"],
            ["collective.solr.port", 8983],
            ["collective.solr.base", "/solr/plone"],
        ],
    )
    def test_registry_keys(self, portal, key, expected):
        """Test if registry keys are set."""
        value = api.portal.get_registry_record(key, default=None)
        assert value == expected

    @pytest.mark.parametrize(
        "item",
        [
            "UID",
            "Title",
            "Description",
            "Type",
            "effective",
            "start",
            "created",
            "end",
            "path_string",
            "mime_type",
            "location",
            "image_scales",
            "image_field",
        ],
    )
    def test_solr_config_fieldlist(self, portal, item):
        key = "kitconcept.solr.config"
        values = api.portal.get_registry_record(key)["fieldList"]
        assert item in values

    def test_solr_config_search_tabs(self, portal):
        key = "kitconcept.solr.config"
        config = api.portal.get_registry_record(key, default=None)
        values = config["searchTabs"]
        assert len(values) == 5
        assert values[0]["label"] == "All"
        assert values[0]["filter"] == "Type(*)"



from base64 import b64decode
from collections import defaultdict
from plone import api
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.namedfile import NamedBlobImage
from plone.restapi.testing import RelativeSession
from zope.component.hooks import setSite

import pytest
import transaction


@pytest.fixture
def users() -> list:
    """Additional users to be created."""
    return []


@pytest.fixture
def contents() -> list:
    """Content to be created."""
    return [
        {
            "_container": "",
            "type": "Image",
            "id": "noamchomsky",
            "title": "Prof. Dr. Noam Chomsky",
            "subjects": ["mymembersubject", "mymembersubjecttwo"],
            "_image": b"iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAAAXNSR0IArs4c6QAAAA1JREFUGFdjCDO+/R8ABKsCZD++CcMAAAAASUVORK5CYII=",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "mydocument",
            "title": "My Document about Noam Chomsky",
        },
        {
            "_container": "",
            "type": "News Item",
            "id": "mynews",
            "title": "My News Item with Noam Chomsky",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "myotherfolder",
            "title": "My other Folder",
        },
    ]


@pytest.fixture
def all_path_string():
    """Helper fixture to extract path information from Solr result."""

    def func(data: dict) -> list[str]:
        return [item["path_string"] for item in data["response"]["docs"]]

    return func


@pytest.fixture
def create_contents(contents):
    """Helper fixture to create initial content."""

    def func(portal) -> dict:
        ids = defaultdict(list)
        for item in contents:
            container_path = item["_container"]
            container = portal.unrestrictedTraverse(container_path)
            payload = {"container": container, "language": "en"}
            if "_image" in item:
                payload["image"] = NamedBlobImage(b64decode(item["_image"]))
            for key, value in item.items():
                if key.startswith("_"):
                    continue
                payload[key] = value
            content = api.content.create(**payload)
            content.language = payload["language"]
            # Set translation
            if "_translation_of" in item:
                source = portal.unrestrictedTraverse(item["_translation_of"])
                ITranslationManager(source).register_translation(
                    content.language, content
                )
            # Transition items
            if "_transitions" in item:
                transitions = item["_transitions"]
                for transition in transitions:
                    api.content.transition(content, transition=transition)
            ids[container_path].append(content.getId())
        return ids

    return func


@pytest.fixture()
def app(functional):
    return functional["app"]


@pytest.fixture()
def http_request(functional):
    return functional["request"]


@pytest.fixture()
def registry_config() -> dict:
    """Fixture with plone.app.registry settings."""
    return {"collective.solr.active": 1}


@pytest.fixture()
def portal(app, solr_service, http_request, users, registry_config):
    """Plone portal with additional users, and registry configuration set."""
    portal = app["plone"]
    setSite(portal)
    current_values = {}
    with api.env.adopt_roles(["Manager", "Member"]):
        for key, value in registry_config.items():
            current_values[key] = api.portal.get_registry_record(key)
            api.portal.set_registry_record(key, value)
        maintenance = api.content.get_view("solr-maintenance", portal, http_request)
        maintenance.clear()
        # Create additional users
        for user in users:
            api.user.create(**user)
    transaction.commit()
    yield portal
    with api.env.adopt_roles(["Manager"]):
        for key, value in current_values.items():
            api.portal.set_registry_record(key, value)
    transaction.commit()


@pytest.fixture()
def portal_with_content(app, portal, create_contents):
    """Plone portal with initial content."""
    with api.env.adopt_roles(["Manager"]):
        content_ids = create_contents(portal)
    transaction.commit()
    yield portal
    with api.env.adopt_roles(["Manager"]):
        containers = sorted(content_ids.keys(), reverse=True)
        for container_path in containers:
            container = portal.unrestrictedTraverse(container_path)
            container.manage_delObjects(content_ids[container_path])
    transaction.commit()


@pytest.fixture()
def maintenance(portal, http_request):
    """Return browser view for solr maintenance."""
    with api.env.adopt_roles(["Manager"]):
        view = api.content.get_view("solr-maintenance", portal, http_request)
    return view


@pytest.fixture()
def request_factory(portal):
    """Fixture returning a session to call the API."""

    def factory() -> RelativeSession:
        url = portal.absolute_url()
        api_session = RelativeSession(url)
        api_session.headers.update({"Accept": "application/json"})
        return api_session

    return factory


@pytest.fixture()
def anon_request(request_factory):
    """Anonymous API requests."""
    return request_factory()


@pytest.fixture()
def manager_request(request_factory):
    """Manager API requests."""
    request = request_factory()
    request.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
    yield request
    request.auth = ()



import pytest


class TestEndpointMultilingual:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content, manager_request):
        self.portal = portal_with_content
        response = manager_request.get(self.url)
        self.data = response.json()


class TestEndpointMultilingualGlobal(TestEndpointMultilingual):
    """Search without a prefix."""

    url = "/@solr?q=chomsky"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", True),
            ("/plone/myfolder/mydocument", True),
            ("/plone/myfolder/mynews", True),
            ("/plone/myfolder", True),
            ("/plone/myfolder_de", True),
            ("/plone/myfolder_de/mydocument", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointMultilingualDisabled(TestEndpointMultilingual):
    """Search with multilingual results disabled."""

    url = "/@solr?q=chomsky&path_prefix=/myfolder&is_multilingual=false"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", False),
            ("/plone/myfolder/mydocument", True),
            ("/plone/myfolder/mynews", True),
            ("/plone/myfolder", True),
            ("/plone/myfolder_de", False),
            ("/plone/myfolder_de/mydocument", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointMultilingualDisabledLocal(TestEndpointMultilingual):
    """Search with multilingual results disabled."""

    url = "/@solr?q=chomsky&path_prefix=/myfolder_de&is_multilingual=false"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", False),
            ("/plone/myfolder/mydocument", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/myfolder", False),
            ("/plone/myfolder_de", True),
            ("/plone/myfolder_de/mydocument", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected



from collections import defaultdict
from plone import api
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.restapi.testing import RelativeSession
from zope.component.hooks import setSite

import pytest
import transaction


@pytest.fixture()
def app(functional):
    return functional["app"]


@pytest.fixture
def create_contents():
    """Helper fixture to create initial content."""

    def func(portal) -> dict:
        ids = defaultdict(list)
        return ids

    return func


@pytest.fixture()
def portal(app, registry_config):
    """Plone portal with additional users, and registry configuration set."""
    portal = app["plone"]
    setSite(portal)
    current_values = {}
    with api.env.adopt_roles(["Manager", "Member"]):
        for key, value in registry_config.items():
            current_values[key] = api.portal.get_registry_record(key)
            api.portal.set_registry_record(key, value)
    transaction.commit()
    yield portal
    with api.env.adopt_roles(["Manager"]):
        for key, value in current_values.items():
            api.portal.set_registry_record(key, value)
    transaction.commit()


@pytest.fixture()
def portal_with_content(app, portal, create_contents):
    """Plone portal with initial content."""
    with api.env.adopt_roles(["Manager"]):
        content_ids = create_contents(portal)
    transaction.commit()
    yield portal
    with api.env.adopt_roles(["Manager"]):
        containers = sorted(content_ids.keys(), reverse=True)
        for container_path in containers:
            container = portal.unrestrictedTraverse(container_path)
            container.manage_delObjects(content_ids[container_path])
    transaction.commit()


@pytest.fixture()
def request_factory(portal):
    """Fixture returning a session to call the API."""

    def factory() -> RelativeSession:
        url = portal.absolute_url()
        api_session = RelativeSession(url)
        api_session.headers.update({"Accept": "application/json"})
        return api_session

    return factory


@pytest.fixture()
def anon_request(request_factory):
    """Anonymous API requests."""
    return request_factory()


@pytest.fixture()
def manager_request(request_factory):
    """Manager API requests."""
    request = request_factory()
    request.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
    yield request
    request.auth = ()



"""Init and utils."""

from zope.i18nmessageid import MessageFactory

import logging


__version__ = "2.0.0a9"

PACKAGE_NAME = "kitconcept.solr"
_ = MessageFactory(PACKAGE_NAME)

logger = logging.getLogger(PACKAGE_NAME)

config = {}



from kitconcept.solr.services.solr_utils import SolrConfig

import pytest


solr_config = {
    "fieldList": [
        "UID",
        "Title",
        "Description",
        "Type",
        "effective",
        "start",
        "created",
        "end",
        "path_string",
        "phone",
        "email",
        "location",
    ],
    "searchTabs": [
        {
            "label": "All",
            "filter": "Type(*)",
        },
        {
            "label": "Pages",
            "filter": "Type:(Page)",
        },
        {
            "label": "News Items",
            "filter": 'Type:("News Item")',
            "layouts": ["list", "grid"],
            "facetFields": ["contact_phone", "contact_email"],
        },
        {
            "label": "Pages and News Items",
            "filter": 'Type:(Page OR "News Item")',
            "layouts": ["grid"],
            "facetFields": ["contact_email"],
        },
    ],
}


@pytest.fixture()
def registry_config() -> dict:
    """Override registry configuration."""
    return {
        "kitconcept.solr.config": solr_config,
    }


class TestUtils:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content):
        self.portal = portal_with_content
        self.solr_config = SolrConfig()


class TestUtilsLabels(TestUtils):
    def test_labels(self):
        assert self.solr_config.labels == [
            "All",
            "Pages",
            "News Items",
            "Pages and News Items",
        ]


class TestUtilsSelectCondition(TestUtils):
    def test_select_condition(self):
        assert self.solr_config.select_condition(0) == "{!tag=typefilter}Type(*)"
        assert self.solr_config.select_condition(1) == "{!tag=typefilter}Type:(Page)"
        assert (
            self.solr_config.select_condition(2)
            == '{!tag=typefilter}Type:("News Item")'
        )
        assert (
            self.solr_config.select_condition(3)
            == '{!tag=typefilter}Type:(Page OR "News Item")'
        )


class TestUtilsFieldList(TestUtils):
    def test_field_list(self):
        assert (
            self.solr_config.field_list
            == "UID,Title,Description,Type,effective,start,created,end,path_string,phone,email,location"
        )


class TestUtilsSelectLayouts(TestUtils):
    def test_select_layouts(self):
        assert self.solr_config.select_layouts(0) is None
        assert self.solr_config.select_layouts(1) is None
        assert self.solr_config.select_layouts(2) == ["list", "grid"]
        assert self.solr_config.select_layouts(3) == ["grid"]


class TestUtilsSelectFacetFields(TestUtils):
    def test_select_facet_fields(self):
        assert self.solr_config.select_facet_fields(0) == []
        assert self.solr_config.select_facet_fields(1) == []
        assert self.solr_config.select_facet_fields(2) == [
            "contact_phone",
            "contact_email",
        ]
        assert self.solr_config.select_facet_fields(3) == ["contact_email"]



from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD

import pytest


@pytest.fixture
def contents() -> list:
    return [
        {
            "_container": "",
            "type": "Document",
            "id": "document1",
            "title": "My Document about Noam Chomsky",
            "_transitions": ["publish"],
        },
        {
            "_container": "",
            "type": "Document",
            "id": "document2",
            "title": "My Private Document about Noam Chomsky",
        },
    ]


@pytest.fixture
def user_credentials() -> tuple:
    return "user2", "averylongpasswordbutnotthatlong"


@pytest.fixture
def member_as_user1_credentials() -> tuple:
    return "member_as_user1", "averylongpasswordbutnotthatlong"


@pytest.fixture
def users(user_credentials, member_as_user1_credentials) -> list:
    return [
        {
            "username": user_credentials[0],
            "email": "user2@foo.bar",
            "password": user_credentials[1],
            "roles": ["Member"],
        },
        {
            "username": member_as_user1_credentials[0],
            "email": "user2@foo.bar",
            "password": member_as_user1_credentials[1],
            "roles": ["Member"],
        },
    ]


@pytest.fixture
def users_credentials_role(user_credentials, member_as_user1_credentials) -> dict:
    return {
        "manager": (SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        "member": user_credentials,
        "anonymous": None,
        "member_as_user1": member_as_user1_credentials,
    }



import pytest


class TestEndpointDefault:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content, manager_request):
        self.portal = portal_with_content
        response = manager_request.get(self.url)
        self.data = response.json()


class TestEndpointDefaultBaseSearch(TestEndpointDefault):
    url = "/@solr?q=chomsky"

    def test_facet_groups(self):
        # Using default configuration
        assert "response" in self.data
        assert self.data.get("facet_groups") == [
            ["All", 3],
            ["Pages", 1],
            ["Events", 0],
            ["Images", 1],
            ["Files", 0],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/mydocument", True),
            ("/plone/noamchomsky", True),
            ("/plone/mynews", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected

    def test_portal_path(self):
        """portal_path is returned for the client"""
        assert "response" in self.data
        assert self.data.get("portal_path") == "/plone"


class TestEndpointDefaultGroupSelectAll(TestEndpointDefault):
    # "filter": "Type(*)",
    url = "/@solr?q=chomsky&group_select=0"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/mydocument", True),
            ("/plone/noamchomsky", True),
            ("/plone/mynews", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointDefaultGroupSelect1(TestEndpointDefault):
    url = "/@solr?q=chomsky&group_select=1"

    def test_facet_groups(self):
        # Using default configuration
        assert "response" in self.data
        assert self.data.get("facet_groups") == [
            ["All", 3],
            ["Pages", 1],
            ["Events", 0],
            ["Images", 1],
            ["Files", 0],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/mydocument", True),
            ("/plone/noamchomsky", False),
            ("/plone/mynews", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointDefaultGroupSelect3(TestEndpointDefault):
    # "filter": "Type:(Image)",
    url = "/@solr?q=chomsky&group_select=3"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/mydocument", False),
            ("/plone/noamchomsky", True),
            ("/plone/mynews", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointDefaultContext(TestEndpointDefault):
    url = "/myotherfolder/@solr?q=chomsky"

    def test_portal_path(self):
        assert "response" in self.data
        assert self.data.get("portal_path") == "/plone"


class TestEndpointDefaultEmptyQSearch(TestEndpointDefault):
    url = "/@solr?q="

    def test_facet_groups(self):
        # Using default configuration
        assert "response" in self.data
        assert self.data.get("facet_groups") == [
            ["All", 5],
            ["Pages", 2],
            ["Events", 0],
            ["Images", 1],
            ["Files", 0],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/mydocument", True),
            ("/plone/noamchomsky", True),
            ("/plone/mynews", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected

    def test_portal_path(self):
        """portal_path is returned for the client"""
        assert "response" in self.data
        assert self.data.get("portal_path") == "/plone"


class TestEndpointDefaultMissingQSearch(TestEndpointDefault):
    url = "/@solr"

    def test_facet_groups(self):
        # Using default configuration
        assert "response" in self.data
        assert self.data.get("facet_groups") == [
            ["All", 5],
            ["Pages", 2],
            ["Events", 0],
            ["Images", 1],
            ["Files", 0],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/mydocument", True),
            ("/plone/noamchomsky", True),
            ("/plone/mynews", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected

    def test_portal_path(self):
        """portal_path is returned for the client"""
        assert "response" in self.data
        assert self.data.get("portal_path") == "/plone"



import pytest


@pytest.fixture
def contents() -> list:
    return [
        {
            "_container": "",
            "type": "Document",
            "id": "document1",
            "title": "This is a blue document",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "document2",
            "title": "This is a red document",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "AnD_oR_nOt_WHATnot",
            "title": "This is a whatnot document",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "oR",
            "title": "This is another document",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "ANDoR",
            "title": "This is another document",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "ANdOR",
            "title": "This is another document",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "OReo",
            "title": "This is another document",
        },
    ]



import pytest


class TestEndpointContentFields:
    url: str = "@solr"

    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content, manager_request):
        self.portal = portal_with_content
        response = manager_request.get(self.url)
        self.data = response.json()


class TestEndpointContentFieldsId(TestEndpointContentFields):
    url = "/@solr?q=red"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/redandblue", True),
            ("/plone/news1", False),
            ("/plone/news2", False),
            ("/plone/news3", False),
            ("/plone/blue", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected

    def test_count(self):
        assert "response" in self.data
        assert self.data["response"]["numFound"] == 1


class TestEndpointContentFieldsTitle(TestEndpointContentFields):
    url = "/@solr?q=news"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/redandblue", False),
            ("/plone/news1", True),
            ("/plone/news2", True),
            ("/plone/news3", True),
            ("/plone/blue", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected

    def test_count(self):
        assert "response" in self.data
        assert self.data["response"]["numFound"] == 4

    @pytest.mark.parametrize(
        "idx,title,location",
        [
            (0, "News Item 1", ""),
            (1, "News Item 2", ""),
            (2, "News Item 3", "My location"),
            (3, "Blue news item", ""),
        ],
    )
    def test_attributes(self, idx: int, title: str, location: str):
        docs = self.data["response"]["docs"]
        item = docs[idx]
        assert item["Title"] == title
        assert item.get("location", "") == location



import base64
import json
import pytest


def encoded(o):
    return base64.b64encode(json.dumps(o).encode("utf-8")).decode("ascii")


solr_config = {
    "fieldList": [
        "UID",
        "Title",
        "Description",
        "Type",
        "effective",
        "start",
        "created",
        "end",
        "path_string",
        "phone",
        "email",
        "location",
    ],
    "searchTabs": [
        {
            "label": "All",
            "filter": "Type(*)",
        },
        {
            "label": "Pages",
            "filter": "Type:(Page)",
            "facetFields": [
                {
                    "name": "Title",
                    "label": "The title",
                },
                {"name": "Description", "label": "The description"},
            ],
        },
        {
            "label": "News Items",
            "filter": 'Type:("News Item")',
        },
        {
            "label": "Pages and News Items",
            "filter": 'Type:(Page OR "News Item")',
        },
    ],
}


@pytest.fixture()
def registry_config() -> dict:
    """Override registry configuration."""
    return {
        "collective.solr.active": 1,
        "kitconcept.solr.config": solr_config,
    }


class TestEndpointCustom:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content, manager_request):
        self.portal = portal_with_content
        response = manager_request.get(self.url)
        self.data = response.json()


class TestFacetConditionsInactive(TestEndpointCustom):
    url = "/@solr?q=chomsky"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/myimage", True),
            ("/plone/myfolder", True),
            ("/plone/myfolder/mynews", True),
            ("/plone/foo_alpha", True),
            ("/plone/foo_beta", True),
            ("/plone/foo_gamma", True),
            ("/plone/bar_alpha", True),
            ("/plone/bar_beta", True),
            ("/plone/bar_gamma", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestFacetConditionsActive(TestEndpointCustom):
    url = "/@solr?q=chomsky&group_select=1"

    def test_facet_fields(self):
        assert self.data.get("facet_fields") == [
            [
                {"name": "Title", "label": "The title"},
                [["chomsky", 6], ["bar", 3], ["foo", 3]],
            ],
            [
                {"name": "Description", "label": "The description"},
                [["chomsky", 6], ["alpha", 2], ["beta", 2], ["gamma", 2]],
            ],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/myimage", False),
            ("/plone/myfolder", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/foo_alpha", True),
            ("/plone/foo_beta", True),
            ("/plone/foo_gamma", True),
            ("/plone/bar_alpha", True),
            ("/plone/bar_beta", True),
            ("/plone/bar_gamma", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestFacetConditionsFiltering1(TestEndpointCustom):
    @property
    def c(self):
        """Conditions with beginning search."""
        return {"Title": {"c": {"foo": True}}}

    @property
    def url(self):
        """URL to test the facets."""
        return f"/@solr?q=chomsky&group_select=1&facet_conditions={encoded(self.c)}"

    def test_facet_fields(self):
        assert self.data.get("facet_fields") == [
            [
                {"name": "Title", "label": "The title"},
                [["chomsky", 6], ["bar", 3], ["foo", 3]],
            ],
            [
                {"name": "Description", "label": "The description"},
                [["chomsky", 6], ["alpha", 2], ["beta", 2], ["gamma", 2]],
            ],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/myimage", False),
            ("/plone/myfolder", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/foo_alpha", True),
            ("/plone/foo_beta", True),
            ("/plone/foo_gamma", True),
            ("/plone/bar_alpha", False),
            ("/plone/bar_beta", False),
            ("/plone/bar_gamma", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestFacetConditionsFiltering2(TestEndpointCustom):
    @property
    def c(self):
        """Conditions with beginning search."""
        return {"Title": {"c": {"foo": True}}, "Description": {"c": {"alpha": True}}}

    @property
    def url(self):
        """URL to test the facets."""
        return f"/@solr?q=chomsky&group_select=1&facet_conditions={encoded(self.c)}"

    def test_facet_fields(self):
        assert self.data.get("facet_fields") == [
            [
                {"name": "Title", "label": "The title"},
                [["chomsky", 6], ["bar", 3], ["foo", 3]],
            ],
            [
                {"name": "Description", "label": "The description"},
                [["chomsky", 6], ["alpha", 2], ["beta", 2], ["gamma", 2]],
            ],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/myimage", False),
            ("/plone/myfolder", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/foo_alpha", True),
            ("/plone/foo_beta", False),
            ("/plone/foo_gamma", False),
            ("/plone/bar_alpha", False),
            ("/plone/bar_beta", False),
            ("/plone/bar_gamma", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestFacetConditionsFiltering3(TestEndpointCustom):
    @property
    def c(self):
        """Conditions with beginning search."""
        return {
            "Title": {"c": {"foo": True}},
            "Description": {"c": {"alpha": True, "beta": True}},
        }

    @property
    def url(self):
        """URL to test the facets."""
        return f"/@solr?q=chomsky&group_select=1&facet_conditions={encoded(self.c)}"

    def test_facet_fields(self):
        assert self.data.get("facet_fields") == [
            [
                {"name": "Title", "label": "The title"},
                [["chomsky", 6], ["bar", 3], ["foo", 3]],
            ],
            [
                {"name": "Description", "label": "The description"},
                [["chomsky", 6], ["alpha", 2], ["beta", 2], ["gamma", 2]],
            ],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/myimage", False),
            ("/plone/myfolder", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/foo_alpha", True),
            ("/plone/foo_beta", True),
            ("/plone/foo_gamma", False),
            ("/plone/bar_alpha", False),
            ("/plone/bar_beta", False),
            ("/plone/bar_gamma", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestFacetConditionsFilteringFalseIgnored(TestEndpointCustom):
    @property
    def c(self):
        """Conditions with beginning search."""
        return {
            "Title": {"c": {"foo": True, "bar": False}},
            "Description": {"c": {"alpha": False}},
        }

    @property
    def url(self):
        """URL to test the facets."""
        return f"/@solr?q=chomsky&group_select=1&facet_conditions={encoded(self.c)}"

    def test_facet_fields(self):
        assert self.data.get("facet_fields") == [
            [
                {"name": "Title", "label": "The title"},
                [["chomsky", 6], ["bar", 3], ["foo", 3]],
            ],
            [
                {"name": "Description", "label": "The description"},
                [["chomsky", 6], ["alpha", 2], ["beta", 2], ["gamma", 2]],
            ],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/myimage", False),
            ("/plone/myfolder", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/foo_alpha", True),
            ("/plone/foo_beta", True),
            ("/plone/foo_gamma", True),
            ("/plone/bar_alpha", False),
            ("/plone/bar_beta", False),
            ("/plone/bar_gamma", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestFacetConditionsContainsBeginning(TestEndpointCustom):
    @property
    def c(self):
        """Conditions with beginning search."""
        return {"Title": {"c": {"foo": True}, "p": "ba"}, "Description": {"p": "cho"}}

    @property
    def url(self):
        """URL to test the facets."""
        return f"/@solr?q=chomsky&group_select=1&facet_conditions={encoded(self.c)}"

    def test_facet_fields(self):
        assert self.data.get("facet_fields") == [
            [
                {"name": "Title", "label": "The title"},
                [["bar", 3]],
            ],
            [
                {"name": "Description", "label": "The description"},
                [["chomsky", 6]],
            ],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/myimage", False),
            ("/plone/myfolder", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/foo_alpha", True),
            ("/plone/foo_beta", True),
            ("/plone/foo_gamma", True),
            ("/plone/bar_alpha", False),
            ("/plone/bar_beta", False),
            ("/plone/bar_gamma", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestFacetConditionsContainsMiddle(TestEndpointCustom):
    @property
    def c(self):
        """Conditions with middle search."""
        return {"Title": {"c": {"foo": True}, "p": "ar"}, "Description": {"p": "cho"}}

    @property
    def url(self):
        """URL to test the facets."""
        return f"/@solr?q=chomsky&group_select=1&facet_conditions={encoded(self.c)}"

    def test_facet_fields(self):
        assert self.data.get("facet_fields") == [
            [
                {"name": "Title", "label": "The title"},
                [["bar", 3]],
            ],
            [
                {"name": "Description", "label": "The description"},
                [["chomsky", 6]],
            ],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/myimage", False),
            ("/plone/myfolder", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/foo_alpha", True),
            ("/plone/foo_beta", True),
            ("/plone/foo_gamma", True),
            ("/plone/bar_alpha", False),
            ("/plone/bar_beta", False),
            ("/plone/bar_gamma", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestFacetConditionsContainsCaseInsensitive(TestEndpointCustom):
    @property
    def c(self):
        """Conditions with case-insensitive search."""
        return {
            "Title": {"c": {"foo": True}, "p": "BAR"},
            "Description": {"p": "cho"},
        }

    @property
    def url(self):
        """URL to test the facets."""
        return f"/@solr?q=chomsky&group_select=1&facet_conditions={encoded(self.c)}"

    def test_facet_fields(self):
        assert self.data.get("facet_fields") == [
            [
                {"name": "Title", "label": "The title"},
                [["bar", 3]],
            ],
            [
                {"name": "Description", "label": "The description"},
                [["chomsky", 6]],
            ],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/myimage", False),
            ("/plone/myfolder", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/foo_alpha", True),
            ("/plone/foo_beta", True),
            ("/plone/foo_gamma", True),
            ("/plone/bar_alpha", False),
            ("/plone/bar_beta", False),
            ("/plone/bar_gamma", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected



"""Module where all interfaces, events and exceptions live."""

from kitconcept.solr import _
from plone.schema import JSONField
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

import json


class IKitconceptSolrLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


CONFIG_SCHEMA = json.dumps({
    "type": "object",
    "properties": {
        "fieldList": {
            "type": "array",
            "items": {"type": "string"},
        },
        "searchTabs": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "label": {"type": "string"},
                    "filter": {"type": "string"},
                },
            },
        },
    },
})

DEFAULT_CONFIG = {
    "fieldList": [
        "UID",
        "Title",
        "Description",
        "Type",
        "effective",
        "start",
        "created",
        "end",
        "path_string",
        "mime_type",
        "phone",
        "email",
        "location",
        "image_scales",
        "image_field",
    ],
    "searchTabs": [
        {"label": "All", "filter": "Type(*)"},
        {"label": "Pages", "filter": "Type:(Page)"},
        {"label": "Events", "filter": "Type:(Event)"},
        {"label": "Images", "filter": "Type:(Image)"},
        {"label": "Files", "filter": "Type:(File)"},
    ],
}


class IKitconceptSolrSettings(Interface):
    config = JSONField(
        title=_("label_solr_config", default="Solr Config"),
        description=_("help_solr_config", default="Solr endpoint configuration"),
        required=True,
        schema=CONFIG_SCHEMA,
        default=DEFAULT_CONFIG,
        missing_value={"fieldList": [], "searchTabs": []},
    )



"""Update locales."""

from pathlib import Path

import logging
import re
import subprocess


logger = logging.getLogger("i18n")
logger.setLevel(logging.DEBUG)


PATTERN = r"^[a-z]{2}.*"

locale_path = Path(__file__).parent.resolve()
target_path = locale_path.parent.resolve()
domains = [path.name[:-4] for path in locale_path.glob("*.pot")]

i18ndude = "uvx i18ndude"

# ignore node_modules files resulting in errors
excludes = '"*.html *json-schema*.xml"'


def locale_folder_setup(domain: str):
    languages = [path for path in locale_path.glob("*") if path.is_dir()]
    for lang_folder in languages:
        lc_messages_path = lang_folder / "LC_MESSAGES"
        lang = lang_folder.name
        if lc_messages_path.exists():
            continue
        elif re.match(PATTERN, lang):
            lc_messages_path.mkdir()
            cmd = (
                f"msginit --locale={lang} "
                f"--input={locale_path}/{domain}.pot "
                f"--output={locale_path}/{lang}/LC_MESSAGES/{domain}.po"
            )
            subprocess.call(cmd, shell=True)  # noQA: S602


def _rebuild(domain: str):
    cmd = (
        f"{i18ndude} rebuild-pot --pot {locale_path}/{domain}.pot "
        f"--exclude {excludes} "
        f"--create {domain} {target_path}"
    )
    subprocess.call(cmd, shell=True)  # noQA: S602


def _sync(domain: str):
    cmd = (
        f"{i18ndude} sync --pot {locale_path}/{domain}.pot "
        f"{locale_path}/*/LC_MESSAGES/{domain}.po"
    )
    subprocess.call(cmd, shell=True)  # noQA: S602


def main():
    for domain in domains:
        logger.info(f"Updating translations for {domain}")
        locale_folder_setup(domain)
        _rebuild(domain)
        _sync(domain)


if __name__ == "__main__":
    main()



import pytest


@pytest.fixture
def contents() -> list:
    return [
        {
            "_container": "",
            "type": "News Item",
            "id": "news1",
            "title": "News Item 1",
            "language": "en",
            "location": "",
        },
        {
            "_container": "",
            "type": "News Item",
            "id": "news2",
            "title": "News Item 2",
            "language": "en",
            "location": "",
        },
        {
            "_container": "",
            "type": "News Item",
            "id": "news3",
            "title": "News Item 3",
            "language": "en",
            "location": "My location",
        },
        {
            "_container": "",
            "type": "Document",
            "id": "redandblue",
            "title": "This is a title",
            "language": "en",
        },
        {
            "_container": "",
            "type": "News Item",
            "id": "blue",
            "title": "Blue news item",
            "language": "en",
        },
    ]



from plone.app.testing import setRoles

import pytest
import transaction


@pytest.fixture()
def member_request(request_factory, user_credentials):
    request = request_factory()
    request.auth = user_credentials
    yield request
    request.auth = ()


class TestEndpointRolesAndUsers:
    url = "/@solr?q=chomsky"

    @pytest.fixture
    def api_request(self, request_factory, users_credentials_role, portal, roles):
        def func(role: str) -> dict:
            req = request_factory()
            credentials = users_credentials_role.get(role, None)
            if credentials:
                req.auth = credentials
                # setting the role is explicitly needed for these tests
                setRoles(portal, credentials[0], roles)
                transaction.commit()
            return req

        return func

    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content):
        self.portal = portal_with_content


class TestEndpointRolesAndUsersNoPermission(TestEndpointRolesAndUsers):
    @pytest.mark.parametrize(
        "role,roles,path,expected",
        [
            ("anonymous", ["Anonymous"], "/plone/document2", False),
            (
                "member_as_user1",
                ["user:test_user_1_"],
                "/plone/document2",
                True,
            ),
            ("member_as_user1", ["Reader"], "/plone/document2", True),
        ],
    )
    def test_paths(
        self,
        api_request,
        all_path_string,
        role: str,
        path: str,
        expected: bool,
    ):
        data = api_request(role).get(self.url).json()
        path_strings = all_path_string(data)
        assert (path in path_strings) is expected



from plone import api

import base64
import json
import logging
import re


logger = logging.getLogger("kitconcept.solr")
logger.setLevel(logging.DEBUG)


SPECIAL_CHARS = [
    "+",
    "-",
    "&",
    "|",
    "!",
    "(",
    ")",
    "{",
    "}",
    "[",
    "]",
    "^",
    '"',
    "~",
    "*",
    "?",
    ":",
    "/",
]


def escape(term):
    # Backslash has to escape itself
    term = term.replace("\\", "\\\\")
    # Escape all chars in the list
    for char in SPECIAL_CHARS:
        term = term.replace(char, "\\" + char)
    return term


def replace_colon(term):
    return term.replace(":", "$")


re_replace_reserved = re.compile(r"\b(AND|OR|NOT)\b")


def replace_reserved(query):
    return re_replace_reserved.sub(lambda m: m.group(0).lower(), query)


class SolrConfigError(RuntimeError):
    pass


class SolrConfig:
    config: dict
    filters: list

    def __init__(self):
        self.config = api.portal.get_registry_record("kitconcept.solr.config")
        search_tabs = self.config.get("searchTabs", [])
        self.filters = [item["filter"] for item in search_tabs]
        self.listOflayouts = [item.get("layouts", None) for item in search_tabs]
        self.listOfFacetFields = [item.get("facetFields", []) for item in search_tabs]

    @property
    def labels(self):
        labels = [item.get("label", "") for item in self.config.get("searchTabs", [])]
        if len(labels) == 0:
            raise SolrConfigError(
                "Error parsing solr config, searchTabs either missing or empty"
            )
        invalid_labels = [label for label in labels if not label]
        if invalid_labels:
            raise SolrConfigError(
                "Error parsing solr config, missing label in searchTabs. "
                "Labels are mandatory."
            )
        return labels

    def select_condition(self, group_select: int) -> str:
        filters = self.filters
        base_query = "{!tag=typefilter}"
        condition = filters[group_select]
        return f"{base_query}{condition}"

    @property
    def field_list(self) -> str:
        raw_value = self.config.get("fieldList", [])
        invalid_fields = [item for item in raw_value if "," in item]
        if invalid_fields:
            raise SolrConfigError(
                "Error parsing solr config, fieldList item contains comma (,) "
                "which is prohibited"
            )
        return ",".join(raw_value)

    def select_layouts(self, group_select: int) -> list:
        return self.listOflayouts[group_select]

    def select_facet_fields(self, group_select: int) -> list:
        return self.listOfFacetFields[group_select]


class FacetConditions:
    config: dict
    # The limits are hardcoded ATM and limit_less MUST match the on the client.
    limit_less: int = 5
    limit_more: int = 100

    def __init__(self, config: dict):
        self.config = config

    @classmethod
    def from_encoded(cls, raw: str | None):
        if raw is not None:
            try:
                config = json.loads(base64.b64decode(raw))
            except (UnicodeDecodeError, json.decoder.JSONDecodeError):
                logger.warning("Ignoring invalid base64 encoded string", exc_info=True)
                config = {}
        else:
            config = {}
        return cls(config)

    @staticmethod
    def value_condition(field_name: str, value: str, selected: bool):
        return (
            (f'{field_name}:"{escape(value)}"' if value else f'{field_name}:["" TO *]')
            if selected
            else None
        )

    @classmethod
    def value_conditions(cls, field_name, field):
        return (
            cls.value_condition(field_name, key, value)
            for key, value in field.get("c", {}).items()
            if cls.value_condition(field_name, key, value) is not None
        )

    @classmethod
    def field_condition(cls, field_name, field):
        conditions = list(cls.value_conditions(field_name, field))
        return " OR ".join(f"({item})" for item in conditions) if conditions else ""

    def field_conditions(self):
        return (
            self.field_condition(name, value)
            for name, value in self.config.items()
            if self.field_condition(name, value)
        )

    @property
    def solr(self):
        conditions = list(self.field_conditions())
        if not conditions:
            return ""

        if len(conditions) == 1:
            return conditions[0]

        return " AND ".join(f"({condition})" for condition in conditions)

    @property
    def contains_query(self):
        return {
            f"f.{name}.facet.contains": config.get("p", "")
            for name, config in self.config.items()
            if config.get("p", "")
        }

    def more_query(self, facet_fields, multiplier):
        return {
            f"f.{field['name']}.facet.limit": (
                multiplier * self.limit_more
                if self.config.get(field["name"], {}).get("m", False)
                else multiplier * (self.limit_less + 1)
            )
            for field in facet_fields
        }

    def more_dict(self, facet_fields, multiplier):
        return {
            field["name"]: (
                multiplier * self.limit_more
                if self.config.get(field["name"], {}).get("m", False)
                else multiplier * (self.limit_less + 1)
            )
            for field in facet_fields
        }


def fix_value(v):
    # XXX this should probably fix in the catalog
    return None if v == "None" else v


def get_facet_fields_result(raw_facet_fields_result, facet_fields, more_dict):
    result = []

    for field_def in facet_fields:
        name = field_def["name"]
        full_array = raw_facet_fields_result[name]
        limit = more_dict[name]

        # Pair values with counts, fixing the value
        paired = zip(
            map(fix_value, full_array[::2]),
            full_array[1::2],
            strict=False,
        )

        # Filter out nulls, empty strings, and reverse tokens
        filtered = [
            (value, count)
            for value, count in paired
            if count > 0 and value and not value.startswith("\x01")
        ]

        # Slice to the desired limit
        result.append((field_def, filtered[:limit]))

    return result



from kitconcept.solr.services.solr_utils import FacetConditions
from kitconcept.solr.services.solr_utils import get_facet_fields_result

import base64
import json
import pytest


def encoded(o):
    return base64.b64encode(json.dumps(o).encode("utf-8")).decode("ascii")


class TestUtilsFacetConditionsSolr:
    def test_create(self):
        o = {
            "field1": {"c": {"foo": True, "bar": True}},
            "field2": {"c": {"baz": True, "bae": True}},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.config == o

    def test_create_none(self):
        c = FacetConditions.from_encoded(None)
        assert c.config == {}

    def test_create_ignores_error_unicode(self):
        c = FacetConditions.from_encoded(
            base64.b64encode('{"foo": "Atommüll"}'.encode("latin1")).decode("ascii")
        )
        assert c.config == {}

    def test_create_ignores_error_json(self):
        c = FacetConditions.from_encoded(
            base64.b64encode(b'{"foo": what is this, surely not json?').decode("ascii")
        )
        assert c.config == {}

    @pytest.mark.parametrize(
        "name,value,expected",
        [
            (
                "solr",
                {
                    "field1": {"c": {"foo": True, "bar": True}},
                    "field2": {"c": {"baz": True, "bae": True}},
                },
                '((field1:"foo") OR (field1:"bar")) AND ((field2:"baz") OR (field2:"bae"))',
            ),
            ("empty", {}, ""),
            (
                "ignores_false",
                {
                    "field1": {"c": {"foo": True, "bar": True}},
                    "field2": {"c": {"baz": True, "bae": True, "bax": False}},
                },
                '((field1:"foo") OR (field1:"bar")) AND ((field2:"baz") OR (field2:"bae"))',
            ),
            (
                "collapses_parent_or",
                {
                    "field1": {"c": {"foo": True, "bar": True}},
                    "field2": {"c": {"baz": True}},
                },
                '((field1:"foo") OR (field1:"bar")) AND ((field2:"baz"))',
            ),
            (
                "collapses_parent_or_with_false",
                {
                    "field1": {"c": {"foo": True, "bar": True}},
                    "field2": {"c": {"baz": True, "bae": False}},
                },
                '((field1:"foo") OR (field1:"bar")) AND ((field2:"baz"))',
            ),
            (
                "collapses_parent_and",
                {
                    "field1": {"c": {"foo": True, "bar": True}},
                },
                '(field1:"foo") OR (field1:"bar")',
            ),
            (
                "collapses_parent_and_with_false",
                {
                    "field1": {"c": {"foo": True, "bar": True}},
                    "field2": {"c": {"baz": False}},
                },
                '(field1:"foo") OR (field1:"bar")',
            ),
            (
                "collapses_parent_and_case2",
                {"field1": {"c": {"foo": True, "bar": True}}, "field2": {"c": {}}},
                '(field1:"foo") OR (field1:"bar")',
            ),
            (
                "collapses_parent_and_case3",
                {"field1": {"c": {"foo": True, "bar": True}}, "field2": {}},
                '(field1:"foo") OR (field1:"bar")',
            ),
            (
                "empty_value",
                {
                    "field1": {"c": {"foo": True, "": True}},
                    "field2": {"c": {"baz": True, "": True}},
                },
                '((field1:"foo") OR (field1:["" TO *])) AND ((field2:"baz") OR (field2:["" TO *]))',
            ),
        ],
    )
    def test_solr(self, name: str, value: dict, expected: str):
        c = FacetConditions.from_encoded(encoded(value))
        assert c.solr == expected, f"{name}: Expected {expected} but got {c.solr}"


class TestUtilsFacetConditionsPrefixQuery:
    def test_create(self):
        o = {
            "field1": {"p": "pre1"},
            "field2": {"p": "pre2"},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.config == o

    def test_create_none(self):
        c = FacetConditions.from_encoded(None)
        assert c.config == {}

    def test_prefix(self):
        o = {
            "field1": {"p": "pre1"},
            "field2": {"p": "pre2"},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.contains_query == {
            "f.field1.facet.contains": "pre1",
            "f.field2.facet.contains": "pre2",
        }

    def test_empty_field(self):
        o = {
            "field1": {"p": "pre1"},
            "field2": {},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.contains_query == {"f.field1.facet.contains": "pre1"}

    def test_empty_field_case2(self):
        o = {
            "field1": {"p": "pre1"},
            "field2": {"c": {"baz": True, "": True}},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.contains_query == {"f.field1.facet.contains": "pre1"}

    def test_all_empty(self):
        o = {
            "field2": {"c": {"baz": True, "": True}},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.contains_query == {}


class TestUtilsFacetConditionsMoreQuery:
    def test_create(self):
        o = {
            "field1": {"m": True},
            "field2": {"m": False},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.config == o

    def test_create_none(self):
        c = FacetConditions.from_encoded(None)
        assert c.config == {}

    def test_more(self):
        o = {
            "field1": {"m": True},
            "field2": {"m": False},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.more_query(
            [{"name": "field1"}, {"name": "field2"}, {"name": "field3"}],
            multiplier=2,
        ) == {
            "f.field1.facet.limit": 200,
            "f.field2.facet.limit": 12,
            "f.field3.facet.limit": 12,
        }

    def test_empty_field(self):
        o = {
            "field1": {"m": True},
            "field2": {},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.more_query(
            [{"name": "field1"}, {"name": "field2"}, {"name": "field3"}],
            multiplier=2,
        ) == {
            "f.field1.facet.limit": 200,
            "f.field2.facet.limit": 12,
            "f.field3.facet.limit": 12,
        }

    def test_empty_field_case2(self):
        o = {
            "field1": {"m": True},
            "field2": {"c": {"baz": True, "": True}},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.more_query(
            [{"name": "field1"}, {"name": "field2"}, {"name": "field3"}],
            multiplier=2,
        ) == {
            "f.field1.facet.limit": 200,
            "f.field2.facet.limit": 12,
            "f.field3.facet.limit": 12,
        }

    def test_all_empty(self):
        o = {
            "field2": {"c": {"baz": True, "": True}},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.more_query(
            [{"name": "field1"}, {"name": "field2"}, {"name": "field3"}],
            multiplier=2,
        ) == {
            "f.field1.facet.limit": 12,
            "f.field2.facet.limit": 12,
            "f.field3.facet.limit": 12,
        }


class TestUtilsFacetConditionsMoreDict:
    def test_create(self):
        o = {
            "field1": {"m": True},
            "field2": {"m": False},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.config == o

    def test_create_none(self):
        c = FacetConditions.from_encoded(None)
        assert c.config == {}

    def test_more(self):
        o = {
            "field1": {"m": True},
            "field2": {"m": False},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.more_dict(
            [{"name": "field1"}, {"name": "field2"}, {"name": "field3"}],
            multiplier=2,
        ) == {
            "field1": 200,
            "field2": 12,
            "field3": 12,
        }

    def test_empty_field(self):
        o = {
            "field1": {"m": True},
            "field2": {},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.more_dict(
            [{"name": "field1"}, {"name": "field2"}, {"name": "field3"}],
            multiplier=2,
        ) == {
            "field1": 200,
            "field2": 12,
            "field3": 12,
        }

    def test_empty_field_case2(self):
        o = {
            "field1": {"m": True},
            "field2": {"c": {"baz": True, "": True}},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.more_dict(
            [{"name": "field1"}, {"name": "field2"}, {"name": "field3"}],
            multiplier=2,
        ) == {
            "field1": 200,
            "field2": 12,
            "field3": 12,
        }

    def test_all_empty(self):
        o = {
            "field2": {"c": {"baz": True, "": True}},
        }
        c = FacetConditions.from_encoded(encoded(o))
        assert c.more_dict(
            [{"name": "field1"}, {"name": "field2"}, {"name": "field3"}],
            multiplier=2,
        ) == {
            "field1": 12,
            "field2": 12,
            "field3": 12,
        }


class TestUtilsGetFacetFieldsResult:
    def test_field_results(self):
        raw_facet_fields_result = {
            "field1": ["foo", 3, "bar", 2],
            "field2": ["baz", 1],
            "field3": ["ignored", 1],
        }
        facet_fields = [
            {"name": "field1", "label": "x"},
            {"name": "field2", "label": "y"},
        ]
        more_dict = {"field1": 5, "field2": 5}
        assert get_facet_fields_result(
            raw_facet_fields_result, facet_fields, more_dict
        ) == [
            ({"name": "field1", "label": "x"}, [("foo", 3), ("bar", 2)]),
            ({"name": "field2", "label": "y"}, [("baz", 1)]),
        ]

    def test_field_results_ignores_zero_count(self):
        raw_facet_fields_result = {
            "field1": ["foo", 3, "bar", 2, "baz", 0],
            "field2": ["baz", 1],
            "field3": ["ignored", 1],
        }
        facet_fields = [
            {"name": "field1", "label": "x"},
            {"name": "field2", "label": "y"},
        ]
        more_dict = {"field1": 5, "field2": 5}
        assert get_facet_fields_result(
            raw_facet_fields_result, facet_fields, more_dict
        ) == [
            ({"name": "field1", "label": "x"}, [("foo", 3), ("bar", 2)]),
            ({"name": "field2", "label": "y"}, [("baz", 1)]),
        ]

    def test_field_results_ignores_empty_value(self):
        raw_facet_fields_result = {
            "field1": ["foo", 3, "bar", 2, "", 12],
            "field2": ["baz", 1],
            "field3": ["ignored", 1],
        }
        facet_fields = [
            {"name": "field1", "label": "x"},
            {"name": "field2", "label": "y"},
        ]
        more_dict = {"field1": 5, "field2": 5}
        assert get_facet_fields_result(
            raw_facet_fields_result, facet_fields, more_dict
        ) == [
            ({"name": "field1", "label": "x"}, [("foo", 3), ("bar", 2)]),
            ({"name": "field2", "label": "y"}, [("baz", 1)]),
        ]

    def test_field_results_ignores_none_value(self):
        raw_facet_fields_result = {
            "field1": ["foo", 3, "bar", 2, None, 12],
            "field2": ["baz", 1],
            "field3": ["ignored", 1],
        }
        facet_fields = [
            {"name": "field1", "label": "x"},
            {"name": "field2", "label": "y"},
        ]
        more_dict = {"field1": 5, "field2": 5}
        assert get_facet_fields_result(
            raw_facet_fields_result, facet_fields, more_dict
        ) == [
            ({"name": "field1", "label": "x"}, [("foo", 3), ("bar", 2)]),
            ({"name": "field2", "label": "y"}, [("baz", 1)]),
        ]

    def test_field_results_ignores_reverse_value(self):
        raw_facet_fields_result = {
            "field1": ["foo", 3, "bar", 2, "\x01rab", 12],
            "field2": ["baz", 1],
            "field3": ["ignored", 1],
        }
        facet_fields = [
            {"name": "field1", "label": "x"},
            {"name": "field2", "label": "y"},
        ]
        more_dict = {"field1": 5, "field2": 5}
        assert get_facet_fields_result(
            raw_facet_fields_result, facet_fields, more_dict
        ) == [
            ({"name": "field1", "label": "x"}, [("foo", 3), ("bar", 2)]),
            ({"name": "field2", "label": "y"}, [("baz", 1)]),
        ]

    def test_field_results_moredict_limits(self):
        raw_facet_fields_result = {
            "field1": [
                "foo",
                3,
                "\x01oof",
                3,
                "bar",
                2,
                "\x01rab",
                12,
                "bax",
                1,
            ],
            "field2": ["baz", 1, "baba", 1],
            "field3": ["ignored", 1],
        }
        facet_fields = [
            {"name": "field1", "label": "x"},
            {"name": "field2", "label": "y"},
        ]
        more_dict = {"field1": 3, "field2": 1}
        assert get_facet_fields_result(
            raw_facet_fields_result, facet_fields, more_dict
        ) == [
            (
                {"name": "field1", "label": "x"},
                [("foo", 3), ("bar", 2), ("bax", 1)],
            ),
            ({"name": "field2", "label": "y"}, [("baz", 1)]),
        ]



import pytest


class TestPreventInjection:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content, manager_request):
        self.portal = portal_with_content
        response = manager_request.get(self.url)
        self.data = response.json()


class TestInjectionOfUppercaseOperatorsIsAvoided(TestPreventInjection):
    url = "/@solr?q=red%20NOT%20blue"

    @pytest.mark.parametrize(
        "path,expected",
        [
            # We test that the NOT word is interpreted as a word and not as a logical
            # operator.
            # If it were, then only document2 would be found which would mean an injection
            # is possible.
            ("/plone/document1", True),
            ("/plone/document2", True),
            ("/plone/AnD_oR_nOt_WHATnot", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestMixedCaseOperatorsAreNotLowercased(TestPreventInjection):
    url = "/@solr?q=AnD_oR_nOt_WHATnot"

    @pytest.mark.parametrize(
        "path,expected",
        [
            # Id is string type and case sensitive so we can test that case sensitivity
            # is not stripped from the service.
            ("/plone/AnD_oR_nOt_WHATnot", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestMixedCaseOperatorsAreNotLowercasedStandalone(TestPreventInjection):
    url = "/@solr?q=oR"

    @pytest.mark.parametrize(
        "path,expected",
        [
            # Id is string type and case sensitive so we can test that case sensitivity
            # is not stripped from the service.
            ("/plone/oR", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestMixedCaseOperatorsAreNotLowercasedIfContainedOnly(TestPreventInjection):
    url = "/@solr?q=ANDoR"

    @pytest.mark.parametrize(
        "path,expected",
        [
            # Id is string type and case sensitive so we can test that case sensitivity
            # is not stripped from the service.
            ("/plone/ANDoR", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestMixedCaseOperatorsAreNotLowercasedIfContainedOnly2(TestPreventInjection):
    url = "/@solr?q=ANdOR"

    @pytest.mark.parametrize(
        "path,expected",
        [
            # Id is string type and case sensitive so we can test that case sensitivity
            # is not stripped from the service.
            ("/plone/ANdOR", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestMixedCaseOperatorsAreNotLowercasedIfContainedOnly3(TestPreventInjection):
    url = "/@solr?q=OReo"

    @pytest.mark.parametrize(
        "path,expected",
        [
            # Id is string type and case sensitive so we can test that case sensitivity
            # is not stripped from the service.
            ("/plone/OReo", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestInjectionOfSymbolOperatorsIsAvoided(TestPreventInjection):
    url = "/@solr?q=red%20!%20blue"

    @pytest.mark.parametrize(
        "path,expected",
        [
            # We test that the ! operator is interpreted as a word and not as a logical
            # operator.
            # If it were, then only document2 would be found which would mean an injection
            # is possible.
            ("/plone/document1", True),
            ("/plone/document2", True),
            ("/plone/AnD_oR_nOt_WHATnot", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestInjectionOfUrlComponentsIsAvoided(TestPreventInjection):
    url = "/@solr?q=red%20%26%20blue"

    @pytest.mark.parametrize(
        "path,expected",
        [
            # We test that the encoded form of & cannot in some way turn into a & in the
            # solr query
            # which would allow feeding arbitrary parameters other than q into the
            # solr server.
            ("/plone/document1", True),
            ("/plone/document2", True),
            ("/plone/AnD_oR_nOt_WHATnot", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected



import pytest


@pytest.fixture()
def member_request(request_factory, user_credentials):
    request = request_factory()
    request.auth = user_credentials
    yield request
    request.auth = ()


class TestEndpointPerms:
    url = "/@solr?q=chomsky"

    @pytest.fixture
    def api_request(self, request_factory, users_credentials_role):
        def func(role: str) -> dict:
            req = request_factory()
            credentials = users_credentials_role.get(role, None)
            if credentials:
                req.auth = credentials
            return req

        return func

    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content):
        self.portal = portal_with_content


class TestEndpointPermsAll(TestEndpointPerms):
    @pytest.mark.parametrize(
        "role,path,expected",
        [
            ("anonymous", "/plone/noamchomsky", True),
            ("anonymous", "/plone/mynews", True),
            ("anonymous", "/plone/chomskysdocument", False),
            ("member", "/plone/noamchomsky", True),
            ("member", "/plone/mynews", True),
            ("member", "/plone/chomskysdocument", False),
            ("manager", "/plone/noamchomsky", True),
            ("manager", "/plone/mynews", True),
            ("manager", "/plone/chomskysdocument", True),
        ],
    )
    def test_paths(
        self,
        api_request,
        all_path_string,
        role: str,
        path: str,
        expected: bool,
    ):
        data = api_request(role).get(self.url).json()
        path_strings = all_path_string(data)
        assert (path in path_strings) is expected


class TestEndpointPermsGroup0(TestEndpointPerms):
    """group=All"""

    url = "/@solr?q=chomsky&group_select=0"

    @pytest.mark.parametrize(
        "role,path,expected",
        [
            ("anonymous", "/plone/noamchomsky", True),
            ("anonymous", "/plone/mynews", True),
            ("anonymous", "/plone/chomskysdocument", False),
            ("member", "/plone/noamchomsky", True),
            ("member", "/plone/mynews", True),
            ("member", "/plone/chomskysdocument", False),
            ("manager", "/plone/noamchomsky", True),
            ("manager", "/plone/mynews", True),
            ("manager", "/plone/chomskysdocument", True),
        ],
    )
    def test_paths(
        self,
        api_request,
        all_path_string,
        role: str,
        path: str,
        expected: bool,
    ):
        data = api_request(role).get(self.url).json()
        path_strings = all_path_string(data)
        assert (path in path_strings) is expected


class TestEndpointPermsGroup3(TestEndpointPerms):
    """group=Image OR News Item"""

    url = "/@solr?q=chomsky&group_select=3"

    @pytest.mark.parametrize(
        "role,path,expected",
        [
            ("anonymous", "/plone/noamchomsky", True),
            ("anonymous", "/plone/mynews", True),
            ("anonymous", "/plone/chomskysdocument", False),
            ("member", "/plone/noamchomsky", True),
            ("member", "/plone/mynews", True),
            ("member", "/plone/chomskysdocument", False),
            ("manager", "/plone/noamchomsky", True),
            ("manager", "/plone/mynews", True),
            ("manager", "/plone/chomskysdocument", False),
        ],
    )
    def test_paths(
        self,
        api_request,
        all_path_string,
        role: str,
        path: str,
        expected: bool,
    ):
        data = api_request(role).get(self.url).json()
        path_strings = all_path_string(data)
        assert (path in path_strings) is expected


class TestEndpointPermsGroup4(TestEndpointPerms):
    """group=Page OR News Item"""

    url = "/@solr?q=chomsky&group_select=4"

    @pytest.mark.parametrize(
        "role,path,expected",
        [
            ("anonymous", "/plone/noamchomsky", False),
            ("anonymous", "/plone/mynews", True),
            ("anonymous", "/plone/chomskysdocument", False),
            ("member", "/plone/noamchomsky", False),
            ("member", "/plone/mynews", True),
            ("member", "/plone/chomskysdocument", False),
            ("manager", "/plone/noamchomsky", False),
            ("manager", "/plone/mynews", True),
            ("manager", "/plone/chomskysdocument", True),
        ],
    )
    def test_paths(
        self,
        api_request,
        all_path_string,
        role: str,
        path: str,
        expected: bool,
    ):
        data = api_request(role).get(self.url).json()
        path_strings = all_path_string(data)
        assert (path in path_strings) is expected


class TestEndpointPermsGroup5(TestEndpointPerms):
    """group=Image"""

    url = "/@solr?q=chomsky&group_select=5"

    @pytest.mark.parametrize(
        "role,path,expected",
        [
            ("anonymous", "/plone/noamchomsky", True),
            ("anonymous", "/plone/mynews", False),
            ("anonymous", "/plone/chomskysdocument", False),
            ("member", "/plone/noamchomsky", True),
            ("member", "/plone/mynews", False),
            ("member", "/plone/chomskysdocument", False),
            ("manager", "/plone/noamchomsky", True),
            ("manager", "/plone/mynews", False),
            ("manager", "/plone/chomskysdocument", False),
        ],
    )
    def test_paths(
        self,
        api_request,
        all_path_string,
        role: str,
        path: str,
        expected: bool,
    ):
        data = api_request(role).get(self.url).json()
        path_strings = all_path_string(data)
        assert (path in path_strings) is expected



import pytest


class TestEndpointLanguage:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content, manager_request):
        self.portal = portal_with_content
        response = manager_request.get(self.url)
        self.data = response.json()


class TestEndpointLanguageEN(TestEndpointLanguage):
    url = "/@solr?q=chomsky&lang=en"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", True),
            ("/plone/myfolder/mydocument", True),
            ("/plone/myfolder/mynews", True),
            ("/plone/myfolder", True),
            ("/plone/myfolder_de", False),
            ("/plone/myfolder_de/mydocument", False),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointLanguageDE(TestEndpointLanguage):
    url = "/@solr?q=chomsky&lang=de"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", False),
            ("/plone/myfolder/mydocument", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/myfolder", False),
            ("/plone/myfolder_de", True),
            ("/plone/myfolder_de/mydocument", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointLanguageError(TestEndpointLanguage):
    url = "/@solr?q=chomsky&lang=en&is_multilingual=true"

    @pytest.mark.parametrize(
        "key,message",
        [
            ("type", "BadRequest"),
            (
                "message",
                "Property 'lang` and `is_multilingual` are mutually exclusive",
            ),
        ],
    )
    def test_error_message(self, key: str, message: str):
        assert self.data[key] == message



from AccessControl.SecurityManagement import getSecurityManager
from collective.solr.interfaces import ISolrConnectionManager
from itertools import zip_longest
from kitconcept.solr.services.solr_utils import escape
from kitconcept.solr.services.solr_utils import FacetConditions
from kitconcept.solr.services.solr_utils import get_facet_fields_result
from kitconcept.solr.services.solr_utils import replace_colon
from kitconcept.solr.services.solr_utils import replace_reserved
from kitconcept.solr.services.solr_utils import SolrConfig
from plone import api
from plone.app.multilingual.interfaces import ITranslatable
from plone.app.multilingual.interfaces import ITranslationManager
from plone.base.utils import base_hasattr
from plone.dexterity.content import DexterityContent
from plone.restapi.services import Service
from zExceptions import BadRequest
from zope.component import queryUtility
from zope.interface import alsoProvides
from zope.publisher.http import HTTPRequest

import json
import plone.protect.interfaces
import re


def security_filter():
    user = getSecurityManager().getUser()
    roles = user.getRoles()
    if "Anonymous" in roles:
        return "allowedRolesAndUsers:Anonymous"
    roles = list(roles)
    roles.append("Anonymous")
    if base_hasattr(user, "getGroups"):
        groups = [f"user:{x}" for x in user.getGroups()]
        if groups:
            roles = roles + groups
    roles.append(f"user:{user.getId()}")
    # Roles with spaces need to be quoted
    roles = [
        (f'"{escape(replace_colon(r))}"' if " " in r else escape(replace_colon(r)))
        for r in roles
    ]
    return f"allowedRolesAndUsers:({' OR '.join(roles)})"


re_relative_path = re.compile("^/+")
re_is_excluding = re.compile("/$")


class SolrSearch(Service):
    context: DexterityContent
    request: HTTPRequest
    _facet_conditions: FacetConditions | None = None

    def _language_settings(self) -> tuple[bool, str]:
        lang = self.request.form.get("lang")
        # search in multilingual path_prefix by default - unless lang is specified
        is_multilinqual_txt = self.request.form.get(
            "is_multilingual", "true" if lang is None else "false"
        )
        is_multilingual = is_multilinqual_txt.lower() == "true"
        if lang and is_multilingual:
            raise BadRequest(
                "Property 'lang` and `is_multilingual` are mutually exclusive"
            )
        return is_multilingual, lang

    def _path_prefix(self, portal_path: list[str]) -> str:
        """Get the path prefix for the search."""

        if not (path_prefix := self.request.form.get("path_prefix", "")):
            # The path_prefix can optionally be used, but the root url is
            # used as default, in case path_prefix is undefined.
            context_path_segments = self.context.getPhysicalPath()
            # Acuqire relative path
            path_prefix_segments = context_path_segments[len(portal_path) :]
            path_prefix = "/".join(path_prefix_segments)
        return path_prefix

    def _group_select(self, solr_config: SolrConfig) -> int | None:
        group_select = self.request.form.get("group_select")
        if group_select is not None:
            try:
                group_select = int(group_select)
            except ValueError as exc:
                raise BadRequest(
                    "Property 'group_select` must be an integer, if specified"
                ) from exc
        elif len(solr_config.filters) > 0:
            # By default select group 0 (unless there are no filters defined)
            group_select = 0
        return group_select

    @property
    def facet_conditions(self) -> FacetConditions:
        """Get the facet conditions from the request."""
        if self._facet_conditions is None:
            raw_value = self.request.form.get("facet_conditions")
            self._facet_conditions = FacetConditions.from_encoded(raw_value)
        return self._facet_conditions

    def _facet_fields(
        self, solr_config: SolrConfig, group_select: int | None
    ) -> list[dict]:
        """Get the facet fields to be used in the query."""
        facet_fields = (
            solr_config.select_facet_fields(group_select)
            if group_select is not None
            else []
        )
        return facet_fields

    def _base_query(
        self,
        solr_config: SolrConfig,
        query: str,
        facet_fields: list[dict],
        group_select: int | None,
        start,
        rows,
        sort,
    ) -> dict:
        """Create a base query for the Solr search."""
        # Note that both escaping and lowercasing reserved words is essential
        # against injection attacks.
        # We only lowercase AND, OR, NOT, but keep e.g. and or And
        # Also only lowercase a full word, and keep OReo intact.
        #
        # In addition. support empty search to search all terms, in case this
        # is configured.
        term = f"({escape(replace_reserved(query))})" if query else "*"

        # Search
        #  q: query parameter
        # fq: the "filter query" parameter allows to restrict the search by filtering
        #     the results
        #     we use the fq param to enforce the allowedRolesAndUsers filter criteria
        # hl: enable highlighting (if set to true)
        # hl.fl: specify the fields to highlight, field needs to have stored=true in
        #     solconf.xml
        # hl.fragsize: set in search.py in collective.solr with 'highlight_fragsize'
        #     param
        # fl: the "field list" parameter defines which fields to include in the response
        # facet: enable faceting (if set to true)
        # facet.field: the fields to facet on
        #
        # Importance:
        #
        # - Title * 5
        # - Description * 2
        # - SearchableText * 1
        # - default * 1
        # - body_text * 1
        # - Subject * 1
        # - ID * 0,75
        # - Text Prefix * 0,75
        # - Text Suffix * 0,75
        # - Text Substring * 0,5
        # - searchwords * 1000
        d = {
            "q": (
                f"+(Title:{term}^5 OR Description:{term}^2 OR id:{term}^0.75 "
                f"OR text_prefix:{term}^0.75 OR text_suffix:{term}^0.75 "
                f"OR default:{term} OR body_text:{term} OR SearchableText:{term} "
                f"OR Subject:{term} OR searchwords:({term})^1000) -showinsearch:False"
            ),
            "wt": "json",
            "hl": "true",
            "hl.fl": "content",  # only used for highlighting, field is not indexed
            "fq": [security_filter()],
            "fl": solr_config.field_list,
            "facet": "true",
            "facet.contains.ignoreCase": "true",
            "facet.field": [
                f"{'{!ex=conditionfilter}' if self.facet_conditions.solr else ''}"
                f"{info['name']}"
                for info in facet_fields
            ],
            # XXX TBD: apply extra_fq from the solr_params.py from the development branch
            # XXX targeted towards v3.
            #
            # Without this the caveat is that the followings are not applied:
            #
            # - path_prefix handling
            # - portal_type handling
            # - language filtering and is_multilingual handling - handled, but actually
            #   not passed in from the service caller
            #
        }
        if start is not None:
            d["start"] = start
        if rows is not None:
            d["rows"] = rows
        if sort is not None:
            d["sort"] = sort

        if group_select is not None:
            d["fq"] = d["fq"] + [solr_config.select_condition(group_select)]
            prefix = (
                "{!ex=typefilter,conditionfilter}"
                if self.facet_conditions.solr
                else "{!ex=typefilter}"
            )
            d["facet.query"] = [f"{prefix}{query}" for query in solr_config.filters]
        return d

    def _apply_facet_conditions(self, query: dict, facet_fields: list[dict]) -> dict:
        """Apply the facet conditions to the query."""
        fc = self.facet_conditions
        # Handle facet conditions
        if fc.solr:
            query["fq"] = query["fq"] + ["{!tag=conditionfilter}" + fc.solr]
        query.update(fc.contains_query)
        query.update(fc.more_query(facet_fields, multiplier=2))
        return query

    def search_solr(self, query: dict) -> dict:
        """Search Solr for the given query."""
        # Get the solr connection
        manager = queryUtility(ISolrConnectionManager)
        manager.setSearchTimeout()
        connection = manager.getConnection()
        raw_result = connection.search(**query).read()
        result = json.loads(raw_result)
        return result

    def enhance_result(
        self,
        result: dict,
        solr_config,
        portal_path: str,
        facet_fields,
        keep_full_solr_response: bool,
        group_select,
    ) -> dict:
        """Enhance the Solr search result."""
        # Add portal path to the result. This can be used by
        # the front-end to calculate the @id from the path_string
        # for each item.
        result["portal_path"] = portal_path
        # Add the group counts in order. Needed because the
        # facet_queries dictionary will not preserve key order, when
        # marshalled to the client.
        # Also add the faces labels next to the corresponding counts.
        result["facet_groups"] = list(
            zip_longest(
                solr_config.labels,
                result["facet_counts"]["facet_queries"].values(),
            )
        )
        result["facet_fields"] = get_facet_fields_result(
            result["facet_counts"]["facet_fields"],
            facet_fields,
            self.facet_conditions.more_dict(facet_fields, multiplier=1),
        )
        # Solr response is pruned of the unnecessary parts, unless explicitly requested.
        if not keep_full_solr_response:
            del result["facet_counts"]
        # Embellish result with supplemental information for the front-end
        if group_select is not None:
            layouts = solr_config.select_layouts(group_select)
            result["layouts"] = layouts
        return result

    def reply(self):
        solr_config = SolrConfig()
        form: dict = self.request.form
        # Disable CSRF protection
        if "IDisableCSRFProtection" in dir(plone.protect.interfaces):
            alsoProvides(self.request, plone.protect.interfaces.IDisableCSRFProtection)

        query = form.get("q", "")
        start = form.get("start")
        rows = form.get("rows")
        sort = form.get("sort")
        portal_type = form.get("portal_type")
        keep_full_solr_response = (
            form.get("keep_full_solr_response", "").lower() == "true"
        )

        is_multilingual, lang = self._language_settings()
        portal = api.portal.get()
        portal_path_segments = portal.getPhysicalPath()
        portal_path = "/".join(portal_path_segments)
        path_prefix = self._path_prefix(portal_path_segments)

        group_select = self._group_select(solr_config)
        facet_fields = self._facet_fields(solr_config, group_select)

        d = self._base_query(
            solr_config, query, facet_fields, group_select, start, rows, sort
        )

        if path_prefix:
            is_excluding = re_is_excluding.search(path_prefix)
            if is_multilingual:
                # All the translation folders have to be queried, if the
                # multilingual option is selected. (By default.)
                relative_path = re_relative_path.sub("", path_prefix)
                folder = self.context.unrestrictedTraverse(relative_path)
                if ITranslatable.providedBy(folder):
                    translations = (
                        ITranslationManager(folder).get_translations().values()
                    )
                else:
                    # Untranslated
                    translations = [folder]
                path_list = ("/".join(o.getPhysicalPath()) for o in translations)
            else:
                # Not multilingual. Search stricly in the given path.
                if is_excluding:
                    path_prefix = path_prefix[:-1]
                path_list = [portal_path + path_prefix]
            if is_excluding:
                # Expressions with a trailing / will exclude the
                # parent folder and include everything else.
                path_expr = [f"path_string:{escape(path + '/')}*" for path in path_list]
            else:
                # Expression that include a folder must be split up to
                # two conditions, to avoid `/something` to match
                # '/something-else'.
                path_expr = [
                    expr
                    for path in path_list
                    for expr in (
                        f"path_string:{escape(path)}",
                        f"path_string:{escape(path + '/')}*",
                    )
                ]
            if len(path_expr) > 0:
                d["fq"] = d["fq"] + ["(" + " OR ".join(path_expr) + ")"]
        if portal_type:
            # Convert to Type condition.
            escaped = [f'"{escape(txt)}"' for txt in portal_type]
            d["fq"] = d["fq"] + [f"Type:({' OR '.join(escaped)})"]
        if lang:
            d["fq"] = d["fq"] + ["Language:(" + escape(lang) + ")"]

        d = self._apply_facet_conditions(d, facet_fields)

        # Run search and enhance the result
        result = self.enhance_result(
            self.search_solr(query=d),
            solr_config,
            portal_path,
            facet_fields,
            keep_full_solr_response,
            group_select,
        )
        return result



from kitconcept.solr.services.solr_utils import escape


class TestUtilsEscape:
    def test_escape_plus(self):
        assert escape("+") == "\\+"

    def test_escape_minus(self):
        assert escape("-") == "\\-"

    def test_escape_ampersand(self):
        assert escape("&") == "\\&"

    def test_escape_vertical_line(self):
        assert escape("|") == "\\|"

    def test_escape_exclamation_mark(self):
        assert escape("!") == "\\!"

    def test_escape_left_parenthesis(self):
        assert escape("(") == "\\("

    def test_escape_right_parenthesis(self):
        assert escape(")") == "\\)"

    def test_escape_left_curly_bracket(self):
        assert escape("{") == "\\{"

    def test_escape_right_curly_bracket(self):
        assert escape("}") == "\\}"

    def test_escape_left_bracket(self):
        assert escape("[") == "\\["

    def test_escape_right_bracket(self):
        assert escape("]") == "\\]"

    def test_escape_caret(self):
        assert escape("^") == "\\^"

    def test_escape_double_quote(self):
        assert escape('"') == '\\"'

    def test_escape_tilde(self):
        assert escape("~") == "\\~"

    def test_escape_asterisk(self):
        assert escape("*") == "\\*"

    def test_escape_question_mark(self):
        assert escape("?") == "\\?"

    def test_escape_colon(self):
        assert escape(":") == "\\:"

    def test_escape_slash(self):
        assert escape("~") == "\\~"

    def test_escape_backslash(self):
        assert escape("\\") == "\\\\"

    def test_escape_double_ampersand(self):
        assert escape("&&") == "\\&\\&"

    def test_escape_double_vertical_line(self):
        assert escape("||") == "\\|\\|"

    def test_escape_mixed_example(self):
        assert escape("Foo&Bar*Baz|") == "Foo\\&Bar\\*Baz\\|"



from collective.solr.utils import isActive
from kitconcept.solr.interfaces import IKitconceptSolrLayer
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


try:
    from plone.restapi.interfaces import ISiteEndpointExpander
except ImportError:
    # added in plone.restapi 9.14.0
    class ISiteEndpointExpander(Interface):
        pass


@adapter(Interface, IKitconceptSolrLayer)
@implementer(ISiteEndpointExpander)
class CollectiveSolrExpander:
    """Add collective.solr.active to the @site endpoint"""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, data):
        data["collective.solr.active"] = isActive()



import pytest


class TestEndpointEncoding:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content, manager_request):
        self.portal = portal_with_content
        response = manager_request.get(self.url)
        self.data = response.json()


class TestEndpointEncodingColon(TestEndpointEncoding):
    url = "/@solr?q=something: colon"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", False),
            ("/plone/myfolder/mydocument", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/withcolon", True),
            ("/plone/withspam", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointEncodingColonEncoded(TestEndpointEncoding):
    url = "/@solr?q=something%3a20colon"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", False),
            ("/plone/myfolder/mydocument", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/withcolon", True),
            ("/plone/withspam", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointEncodingSpan(TestEndpointEncoding):
    url = "/@solr?q=something:+-&|!()${'{}'}[]^+~*?/ spam"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", False),
            ("/plone/myfolder/mydocument", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/withcolon", True),
            ("/plone/withspam", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected


class TestEndpointEncodingSpanEncoded(TestEndpointEncoding):
    url = "/@solr?q=something%3A%2B-%26%7C!()%7B%7D%5B%5D%5E%2B~*%3F%2F spam"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/noamchomsky", False),
            ("/plone/myfolder/mydocument", False),
            ("/plone/myfolder/mynews", False),
            ("/plone/withcolon", True),
            ("/plone/withspam", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected



import pytest


solr_config = {
    "fieldList": [
        "UID",
        "Title",
        "Description",
        "Type",
        "effective",
        "start",
        "created",
        "end",
        "path_string",
        "phone",
        "email",
        "location",
    ],
    "searchTabs": [
        {
            "label": "All",
            "filter": "Type(*)",
        },
        {
            "label": "Pages",
            "filter": "Type:(Page)",
        },
        {
            "label": "News Items",
            "filter": 'Type:("News Item")',
        },
        {
            "label": "Pages and News Items",
            "filter": 'Type:(Page OR "News Item")',
        },
    ],
}


@pytest.fixture()
def registry_config() -> dict:
    """Override registry configuration."""
    return {
        "collective.solr.active": 1,
        "kitconcept.solr.config": solr_config,
    }


class TestEndpointCustom:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content, manager_request):
        self.portal = portal_with_content
        response = manager_request.get(self.url)
        self.data = response.json()


class TestEndpointCustomBaseSearch(TestEndpointCustom):
    url = "/@solr?q=chomsky"

    def test_facet_groups(self):
        # Using default configuration
        assert "response" in self.data
        assert self.data.get("facet_groups") == [
            ["All", 3],
            ["Pages", 1],
            ["News Items", 1],
            ["Pages and News Items", 2],
        ]

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/mydocument", True),
            ("/plone/noamchomsky", True),
            ("/plone/mynews", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected

    def test_portal_path(self):
        """portal_path is returned for the client"""
        assert "response" in self.data
        assert self.data.get("portal_path") == "/plone"


class TestEndpointCustomGroupSelect1(TestEndpointCustom):
    # "filter": "Type(Page OR News Item)",
    url = "/@solr?q=chomsky&group_select=3"

    @pytest.mark.parametrize(
        "path,expected",
        [
            ("/plone/mydocument", True),
            ("/plone/noamchomsky", False),
            ("/plone/mynews", True),
        ],
    )
    def test_paths(self, all_path_string, path: str, expected: bool):
        path_strings = all_path_string(self.data)
        assert (path in path_strings) is expected



from .solr import security_filter
from collective.solr.interfaces import ISolrConnectionManager
from kitconcept.solr.services.solr_utils import escape
from kitconcept.solr.services.solr_utils import replace_reserved
from plone import api
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.services import Service
from zExceptions import BadRequest
from zope.component import getMultiAdapter
from zope.component import queryUtility

import json
import urllib


class SolrSuggest(Service):
    def _language_settings(self) -> tuple[bool, str]:
        lang = self.request.form.get("lang")
        # search in multilingual path_prefix by default - unless lang is specified
        is_multilinqual_txt = self.request.form.get(
            "is_multilingual", "true" if lang is None else "false"
        )
        is_multilingual = is_multilinqual_txt.lower() == "true"
        if lang and is_multilingual:
            raise BadRequest(
                "Property 'lang` and `is_multilingual` are mutually exclusive"
            )
        return is_multilingual, lang

    def query_suggest(self, query):
        manager = queryUtility(ISolrConnectionManager)
        if manager is None:
            return {"error": "Solr is not installed or activated"}
        connection = manager.getConnection()
        if connection is None:
            return {"error": "Solr is not installed or activated"}
        data = {"error": "no response"}
        _, lang = self._language_settings()
        term = f"({escape(replace_reserved(query))})" if query else "*"
        d = {
            "q": (
                f"+suggest:{term}^10 OR +suggest_ngram:{term} OR +searchwords:{term}^1000 OR +suggest_searchwords_ngram:{term}"
            ),
            "fq": [
                security_filter(),
                "-showinsearch:False",
                "-portal_type:Image -portal_type:Glossary -portal_type:FAQ -portal_type:(FAQ Item) -portal_type:(FAQ Category) -portal_type:Link",
            ],
            "defType": "lucene",
        }

        if lang:
            d["fq"] = d["fq"] + ["Language:(" + escape(lang) + ")"]

        d["fq"] = " AND ".join(d["fq"])
        querystring = urllib.parse.urlencode(d)
        url = "{}/{}".format(connection.solrBase, "suggest?%s" % querystring)
        try:
            res = connection.doGet(url, {"Accept": "application/json"})
            data = json.loads(res.read())
        finally:
            if not connection.persistent:
                connection.conn.close()
        return data

    def serialize_brain(self, brain):
        if brain["portal_type"] in ["Member"]:
            obj = brain.getObject()
            data = getMultiAdapter((obj, self.request), ISerializeToJson)()
            data["@id"] = obj.absolute_url()
            return data

        return getMultiAdapter((brain, self.request), ISerializeToJsonSummary)()

    def parse_response(self, data):
        if "error" in data or "response" not in data:
            error = {"suggestions": [], "error": "No response from solr"}
            if "error" in data:
                error["error"] = data["error"]
            return error
        uids = [doc["UID"] for doc in data["response"]["docs"]]
        brains = {brain["UID"]: brain for brain in api.content.find(UID=uids)}
        return [self.serialize_brain(brains[uid]) for uid in uids if uid in brains]

    def reply(self):
        query = self.request.form.get("query", "")
        data = self.query_suggest(query)
        data = self.parse_response(data)
        if isinstance(data, dict):
            return data
        return {"suggestions": data}



from kitconcept.solr.services.solr_utils import SolrConfig
from kitconcept.solr.services.solr_utils import SolrConfigError

import pytest


solr_config = {
    "fieldList": [
        "UID",
        "Title",
        "Description",
        "Type",
        "effective",
        "start",
        "created",
        "end",
        "path_string",
        "phone",
        "email",
        "location",
    ],
    "searchTabs": [],
}


@pytest.fixture()
def registry_config() -> dict:
    """Override registry configuration."""
    return {
        "kitconcept.solr.config": solr_config,
    }


class TestUtilsEmptySearchTabs:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content):
        self.portal = portal_with_content
        self.solr_config = SolrConfig()

    def test_empty_search_tabs(self):
        with pytest.raises(SolrConfigError) as exc_info:
            _ = self.solr_config.labels
        assert exc_info.type is SolrConfigError



import pytest
import urllib.parse


class TestSuggestDefault:
    @pytest.fixture(autouse=True)
    def _init(self, portal_with_content, manager_request):
        self.portal = portal_with_content
        response = manager_request.get(self.url)
        self.data = response.json()


@pytest.fixture
def get_suggest_result_props():
    def func(
        a: dict,
    ) -> bool:
        ac = dict(a)
        del ac["@id"]
        return ac

    return func


@pytest.fixture
def get_suggest_result_path():
    def func(a: dict) -> bool:
        return urllib.parse.urlparse(a["@id"]).path

    return func


@pytest.fixture
def get_suggest_item():
    def func(data, index: int) -> dict:
        return data.get("suggestions")[index]

    return func


class TestSuggestDefaultBaseSearch(TestSuggestDefault):
    url = "/@solr-suggest?query=chomsky"
    expected_result = [
        {
            "@id": "http://localhost:59793/plone/mydocument",
            "@type": "Document",
            "description": "",
            "review_state": "private",
            "title": "My Document about Noam Chomsky",
            "type_title": "Page",
        },
        {
            "@id": "http://localhost:59793/plone/mynews",
            "@type": "News Item",
            "description": "",
            "review_state": "private",
            "title": "My News Item with Noam Chomsky",
            "type_title": "News Item",
        },
    ]

    @pytest.mark.parametrize(
        "index,expected_dict",
        enumerate(expected_result),
    )
    def test_suggest_result_path(
        self,
        get_suggest_item,
        get_suggest_result_path,
        index: int,
        expected_dict: dict,
    ):
        assert get_suggest_result_path(
            get_suggest_item(self.data, index)
        ) == get_suggest_result_path(expected_dict)

    @pytest.mark.parametrize(
        "index,expected_dict",
        enumerate(expected_result),
    )
    def test_suggest_result_props(
        self,
        get_suggest_item,
        get_suggest_result_props,
        index: int,
        expected_dict: dict,
    ):
        assert get_suggest_result_props(
            get_suggest_item(self.data, index)
        ) == get_suggest_result_props(expected_dict)



from kitconcept.solr.reindex_helpers import activate_and_reindex
from Testing.makerequest import makerequest
from zope.site.hooks import setSite

import sys
import transaction


if __name__ == "__main__":
    app = makerequest(globals()["app"])

    # Set site to Plone
    site_id = "Plone"
    portal = app.unrestrictedTraverse(site_id)
    setSite(portal)

    activate_and_reindex(portal, clear="--clear" in sys.argv)

    transaction.commit()
    app._p_jar.sync()
